//	Asteroid.h - A basic physically-controlled asteroid.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef ASTEROID_H_
#define ASTEROID_H_

#include "Collidable.h"
#include "TwoFloats.h"

#include "Box2D/Box2D.h"
#include <stdint.h>

class Drawer;

///	A basic physically-controlled asteroid.
class Asteroid : public Collidable
{
  public:
	///	Constructor.
	Asteroid();
	///	Destructor.
	~Asteroid();

	///	Initialises all the physics stuff.
	void initialise(b2World& world, const TwoFloats& pos);

	///	Draws the asteroid.
	void draw(Drawer *d, const TwoFloats& playerPos);

	///	Returns the asteroid's position.
	TwoFloats getPosition() const;
	///	Returns the asteroid's Box2D body.
	b2Body *getBody() {return physicsBody;};

	///	Called when the asteroid enters/leaves a nebula.
	void nebulaCollide(bool val);

	///	For collisions.
	const std::string getName() const {return "Asteroid";};
  private:
	///	The 4 asteroid images.
	static uint32_t images[4];

	///	Which image this asteroid uses.
	int imageIndex;

	///	The asteroid's Box2D body.
	b2Body *physicsBody;
};

#endif
