//	DonkeyHerd.cpp - Classes representing he donkey herd.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "DonkeyHerd.h"
#include "Box2DHelpers.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "NiallsDrawingLib/OpenGLDrawer.h"
#include "Images.h"
#include "Random.h"

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
DonkeyHerd::DonkeyHerd():
spiralIndex(0.0f),
playerDistance(32000.0f),
win(false),
homing(false)
{
	spiralIndex = Random::getGlobal().getRandomFloat() * 6.283185f;
}

//------------------------------------------------------------------------------
DonkeyHerd::~DonkeyHerd()
{

}

//------------------------------------------------------------------------------
void DonkeyHerd::initialise(b2World& world)
{
	int i;
	TwoFloats dir;
	TwoFloats perp;
	TwoFloats tempPos;
	TwoFloats startPos(64000.0f + (sinf(spiralIndex) * playerDistance),
					   32000.0f + (-cosf(spiralIndex) * playerDistance));

	dir = (startPos - TwoFloats(64000.0f, 32000.0f)).normal().perpendicular();

	for(i=0;i<(NumDonkeys/3);++i)
	{
		//The central donkey.
		{
			b2BodyDef bodyDef;
			b2FixtureDef fixtureDef;
			b2PolygonShape boundingBox;
			b2PolygonShape flamesBox;
			b2Body *physicsBody;

			tempPos = startPos;
			tempPos += (float)i * 280.0f * dir;

			bodyDef.type = b2_dynamicBody;
			bodyDef.position.Set(pixels2Meters(tempPos.x), pixels2Meters(tempPos.y));
			physicsBody = world.CreateBody(&bodyDef);

			boundingBox.SetAsBox(pixels2Meters(72.0f), pixels2Meters(128.0f));
			fixtureDef.shape = &boundingBox;
			fixtureDef.density = 1.0f;
			fixtureDef.friction = 0.3f;
			fixtureDef.userData = &(donkeys[i*3]);
			physicsBody->CreateFixture(&fixtureDef);

			physicsBody->SetTransform(bodyDef.position, 1.570796f);
			physicsBody->SetLinearDamping(0.5f);
			physicsBody->SetAngularDamping(0.05f);

			//Attach to the previous donkey if we're not the front one.
			if(i > 0)
			{
				//b2DistanceJointDef jointDef;
				b2RopeJointDef jointDef;
				b2Body *prevBody = donkeys[(i-1)*3].getBody();

				/*jointDef.Initialize(prevBody,
									physicsBody,
									prevBody->GetPosition(),
									physicsBody->GetPosition());
				jointDef.length = 8.0f;*/
				jointDef.bodyA = prevBody;
				jointDef.bodyB = physicsBody;
				jointDef.localAnchorA = b2Vec2(0.0f, 1.0f);
				jointDef.localAnchorB = b2Vec2(0.0f, -1.0f);
				jointDef.maxLength = 16.0f;
				jointDef.collideConnected = true;

				world.CreateJoint(&jointDef);
			}

			donkeys[(i*3)].initialise(physicsBody);
		}

		//The left donkey.
		{
			b2BodyDef bodyDef;
			b2FixtureDef fixtureDef;
			b2PolygonShape boundingBox;
			b2PolygonShape flamesBox;
			b2Body *physicsBody;
			//b2DistanceJointDef jointDef;
			b2RopeJointDef jointDef;

			/*tempPos.x -= 64.0f;
			tempPos.y -= 192.0f;*/
			perp = tempPos.perpendicular().normal();
			tempPos -= (perp * 256.0f);

			bodyDef.type = b2_dynamicBody;
			bodyDef.position.Set(pixels2Meters(tempPos.x), pixels2Meters(tempPos.y));
			physicsBody = world.CreateBody(&bodyDef);

			boundingBox.SetAsBox(pixels2Meters(72.0f), pixels2Meters(128.0f));
			fixtureDef.shape = &boundingBox;
			fixtureDef.density = 1.0f;
			fixtureDef.friction = 0.3f;
			fixtureDef.userData = &(donkeys[(i*3)+1]);
			physicsBody->CreateFixture(&fixtureDef);

			physicsBody->SetTransform(bodyDef.position, 1.570796f);
			physicsBody->SetLinearDamping(0.5f);
			physicsBody->SetAngularDamping(0.05f);

			/*jointDef.Initialize(donkeys[(i*3)].getBody(),
								physicsBody,
								donkeys[(i*3)].getBody()->GetPosition(),
								physicsBody->GetPosition());
			jointDef.length = 6.0f;*/
			jointDef.bodyA = donkeys[(i*3)].getBody();
			jointDef.bodyB = physicsBody;
			jointDef.localAnchorA = b2Vec2(0.0f, 1.0f);
			jointDef.localAnchorB = b2Vec2(0.0f, 1.0f);
			jointDef.maxLength = 12.0f;
			jointDef.collideConnected = true;

			world.CreateJoint(&jointDef);

			donkeys[(i*3)+1].initialise(physicsBody);
		}

		//The right donkey.
		{
			b2BodyDef bodyDef;
			b2FixtureDef fixtureDef;
			b2PolygonShape boundingBox;
			b2PolygonShape flamesBox;
			b2Body *physicsBody;
			//b2DistanceJointDef jointDef;
			b2RopeJointDef jointDef;

			//tempPos.y += (192.0f * 2.0f);
			tempPos += (perp * 256.0f * 2.0f);

			bodyDef.type = b2_dynamicBody;
			bodyDef.position.Set(pixels2Meters(tempPos.x), pixels2Meters(tempPos.y));
			physicsBody = world.CreateBody(&bodyDef);

			boundingBox.SetAsBox(pixels2Meters(72.0f), pixels2Meters(128.0f));
			fixtureDef.shape = &boundingBox;
			fixtureDef.density = 1.0f;
			fixtureDef.friction = 0.3f;
			fixtureDef.userData = &(donkeys[(i*3)+2]);
			physicsBody->CreateFixture(&fixtureDef);

			physicsBody->SetTransform(bodyDef.position, 1.570796f);
			physicsBody->SetLinearDamping(0.5f);
			physicsBody->SetAngularDamping(0.05f);

			/*jointDef.Initialize(donkeys[(i*3)].getBody(),
								physicsBody,
								donkeys[(i*3)].getBody()->GetPosition(),
								physicsBody->GetPosition());
			jointDef.length = 6.0f;*/
			jointDef.bodyA = donkeys[(i*3)].getBody();
			jointDef.bodyB = physicsBody;
			jointDef.localAnchorA = b2Vec2(0.0f, 1.0f);
			jointDef.localAnchorB = b2Vec2(0.0f, 1.0f);
			jointDef.maxLength = 12.0f;
			jointDef.collideConnected = true;

			world.CreateJoint(&jointDef);

			donkeys[(i*3)+2].initialise(physicsBody);
		}
	}

	//Setup the mouse joint we use to drag the herd around.
	{
		b2MouseJointDef jointDef;

		jointDef.bodyA = donkeys[0].getBody();
		jointDef.bodyB = donkeys[3].getBody();
		jointDef.target = b2Vec2(pixels2Meters(startPos.x), pixels2Meters(startPos.y));
		jointDef.collideConnected = true;
		jointDef.maxForce = 12000.0f;

		mouseJoint = (b2MouseJoint *)world.CreateJoint(&jointDef);
	}
}

//------------------------------------------------------------------------------
void DonkeyHerd::move(const TwoFloats& playerPos)
{
	TwoFloats newPos;
	TwoFloats firstDonkeyPos = donkeys[0].getPosition();
	float tempDist = firstDonkeyPos.getDistance(playerPos);

	if(win)
	{
		spiralIndex += 0.01f;

		newPos.x = winPoint.x + (sinf(spiralIndex) * 4000.0f);
		newPos.y = winPoint.y + (-cosf(spiralIndex) * 4000.0f);

		mouseJoint->SetTarget(b2Vec2(pixels2Meters(newPos.x), pixels2Meters(newPos.y)));
	}
	else if(homing)
	{
		newPos = 0.0f - (firstDonkeyPos - playerPos).normal();
		if(tempDist > 4096.0f)
			newPos *= 4096.0f;
		else
			newPos *= tempDist;
		newPos += firstDonkeyPos;

		mouseJoint->SetTarget(b2Vec2(pixels2Meters(newPos.x), pixels2Meters(newPos.y)));
	}
	else
	{
		spiralIndex += 0.005f;

		/*if(tempDist < playerDistance)
			playerDistance = tempDist;
		else */if(playerDistance > 50.0f)
			playerDistance -= 0.25f;

		newPos.x = playerPos.x + (sinf(spiralIndex) * playerDistance);
		newPos.y = playerPos.y + (-cosf(spiralIndex) * playerDistance);

		mouseJoint->SetTarget(b2Vec2(pixels2Meters(newPos.x), pixels2Meters(newPos.y)));
	}
}

//------------------------------------------------------------------------------
void DonkeyHerd::draw(Drawer *d, const TwoFloats& playerPos)
{
	int i;
	TwoFloats topLeft, bottomRight;

	topLeft.x = playerPos.x - 640.0f - 256.0f;
	topLeft.y = playerPos.y - 360.0f - 256.0f;
	bottomRight.x = playerPos.x + 640.0f + 256.0f;
	bottomRight.y = playerPos.y + 360.0f + 256.0f;
	
	/*d->fillRect((playerPos.x-640.0f)/100.0f,
				(playerPos.y-360.0f)/100.0f,
				(1280.0f/100.0f),
				(720.0f/100.0f),
				255,
				255,
				255,
				255);*/

	Donkey::drawStars(d, playerPos);

	for(i=0;i<NumDonkeys;++i)
	{
		TwoFloats pos(donkeys[i].getPosition());

		if(pos.isWithin(topLeft, bottomRight))
			donkeys[i].draw(d, playerPos);

		/*d->fillRect(pos.x/100.0f,
					pos.y/100.0f,
					1,
					1,
					255, 255, 0, 255);*/
	}
	/*d->fillRect(playerPos.x/100.0f,
				playerPos.y/100.0f,
					2,
					2,
					0, 0, 0, 255);*/

	if(0)
	{
		TwoFloats dir = donkeys[0].getPosition();

		dir = (dir-playerPos).normal();

		d->fillRect(640.0f + (dir.x * 100.0f) - 12,
					360.0f + (dir.y * 100.0f) - 12,
					24,
					24,
					255,
					255,
					255,
					255);
		d->fillRect(640.0f + (dir.x * 100.0f) - 10,
					360.0f + (dir.y * 100.0f) - 10,
					20,
					20,
					0,
					0,
					0,
					255);
	}
}

//------------------------------------------------------------------------------
void DonkeyHerd::gameWin(const TwoFloats& playerPos)
{
	win = true;

	spiralIndex = (Random::getGlobal().getRandomFloat() * 359.99f);
	winPoint.x = playerPos.x + (sinf(spiralIndex) * 4000.0f);
	winPoint.y = playerPos.y - (cosf(spiralIndex) * 4000.0f);
}

//------------------------------------------------------------------------------
void DonkeyHerd::setHoming()
{
	homing = true;
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
uint32_t DonkeyHerd::Donkey::image = 0;
uint32_t DonkeyHerd::Donkey::flameImages[4] = {0, 0, 0, 0};
StarParticles DonkeyHerd::Donkey::stars;

//------------------------------------------------------------------------------
DonkeyHerd::Donkey::Donkey():
starsCount(0),
flameIndex(0),
flameCount(5)
{
			
}

//------------------------------------------------------------------------------
DonkeyHerd::Donkey::~Donkey()
{

}

//------------------------------------------------------------------------------
void DonkeyHerd::Donkey::initialise(b2Body *body)
{
	physicsBody = body;
}

//------------------------------------------------------------------------------
void DonkeyHerd::Donkey::draw(Drawer *d, const TwoFloats& playerPos)
{
	b2Vec2 pos = physicsBody->GetPosition();
	TwoFloats donkeyPos(meters2Pixels(pos.x), meters2Pixels(pos.y));

	if(!image)
	{
		ImageData data;
		ImageLoader imLoad;
		OpenGLDrawer *openGLD = dynamic_cast<OpenGLDrawer *>(d);

		if(openGLD)
			openGLD->setTexInterp(false);

		data = imLoad.loadImage(Images::donkey_png, Images::donkey_pngSize);
		image = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::bottomfire0_png, Images::bottomfire0_pngSize);
		flameImages[0] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::bottomfire1_png, Images::bottomfire1_pngSize);
		flameImages[1] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::bottomfire2_png, Images::bottomfire2_pngSize);
		flameImages[2] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::bottomfire3_png, Images::bottomfire3_pngSize);
		flameImages[3] = d->loadImage(data.data, data.width, data.height);

		if(openGLD)
			openGLD->setTexInterp(true);
	}

	//stars.draw(d, playerPos);

	//Draw flames.
	{
		float angle = physicsBody->GetAngle();
		TwoFloats dir(sinf(angle), -cosf(angle));
		TwoFloats tempPos(donkeyPos);

		tempPos.x -= (playerPos.x-640.0f);
		tempPos.y -= (playerPos.y-360.0f);

		tempPos -= (dir*88.0f);
		tempPos += (dir.perpendicular()*16.0f*4.0f);

		d->drawImageRot(flameImages[flameIndex],
						tempPos.x,
						tempPos.y+16.0f,
						angle * (180.0f/3.14159265f),
						0.0f,
						0.0f,
						4.0f,
						4.0f);

		--flameCount;
		if(flameCount <= 0)
		{
			flameCount = 5;
			++flameIndex;
			flameIndex %= 4;
		}

		--starsCount;
		if(starsCount <= 0)
		{
			TwoFloats starPos(getPosition());

			starsCount = 5 + (int)(Random::getGlobal().getRandomFloat() * 4.99f);

			starPos -= (dir*96.0f);
			starPos.x += (Random::getGlobal().getRandomFloat() * 32.0f)-16.0f;
			starPos.y += (Random::getGlobal().getRandomFloat() * 32.0f)-16.0f;
			stars.addStar(starPos);
		}
	}

	d->drawImageRot(image,
					donkeyPos.x-(playerPos.x-640.0f)-128.0f,
					donkeyPos.y-(playerPos.y-360.0f)-128.0f,
					physicsBody->GetAngle() * (180.0f/3.14159265f),
					32.0f,
					32.0f,
					4.0f,
					4.0f);
};

//------------------------------------------------------------------------------
const TwoFloats DonkeyHerd::Donkey::getPosition() const
{
	TwoFloats retval;
	b2Vec2 physicsPos = physicsBody->GetPosition();

	retval.x = meters2Pixels(physicsPos.x);
	retval.y = meters2Pixels(physicsPos.y);

	return retval;
}

//------------------------------------------------------------------------------
void DonkeyHerd::Donkey::drawStars(Drawer *d, const TwoFloats& playerPos)
{
	stars.draw(d, playerPos);
}
