//	DonkeyHerd.h - Classes representing he donkey herd.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef DONKEYHERD_H_
#define DONKEYHERD_H_

#include "Collidable.h"
#include "TwoFloats.h"
#include "StarParticles.h"

#include "Box2D/Box2D.h"
#include <stdint.h>
#include <string>

class Drawer;

//------------------------------------------------------------------------------
///	Class representing the donkey herd.
class DonkeyHerd
{
  public:
	///	Constructor.
	DonkeyHerd();
	///	Destructor.
	~DonkeyHerd();

	///	Initialises the physics stuff.
	void initialise(b2World& world);

	///	Moves the herd (really just moves the herd leader; the rest will follow).
	void move(const TwoFloats& playerPos);
	///	Draws the herd.
	void draw(Drawer *d, const TwoFloats& playerPos);

	///	Makes it so we no longer move around the player.
	void gameWin(const TwoFloats& playerPos);

	///	Returns the position of the first donkey in the herd.
	TwoFloats getPosition() {return donkeys[0].getPosition();};

	///	Makes the herd move directly to the player donkey.
	void setHoming();

	///	Class representing an individual donkey.
	class Donkey : public Collidable
	{
	  public:
		///	Constructor.
		Donkey();
		///	Destructor.
		~Donkey();

		///	Initialises the donkey's physics.
		void initialise(b2Body *body);

		///	Draws the donkey.
		void draw(Drawer *d, const TwoFloats& playerPos);

		///	Returns the donkey's position.
		const TwoFloats getPosition() const;
		///	Returns the donkey's Box2D physics body.
		b2Body *getBody() const {return physicsBody;};

		///	For collisions.
		const std::string getName() const {return "HerdDonkey";};

		///	Draws all the Donkey's stars.
		static void drawStars(Drawer *d, const TwoFloats& playerPos);
	  private:
		///	The donkey image.
		static uint32_t image;
		///	The flames images.
		static uint32_t flameImages[4];

		///	The stars the donkey leaves behind as it flies through space.
		static StarParticles stars;
		///	Used to determine when to add a new star particle.
		int starsCount;

		///	The donkey's Box2D body.
		b2Body *physicsBody;

		///	Used to determine which frame of the fire to display.
		int flameIndex;
		///	Used to determine which frame of the fire to display.
		int flameCount;
	};
  private:
	///	The number of donkeys in the herd.
	enum
	{
		NumDonkeys = 12 * 3 //Must be a multiple of 3.
	};

	///	The herd's donkeys.
	Donkey donkeys[NumDonkeys];

	///	The mouse joint used to drag the herd around.
	b2MouseJoint *mouseJoint;

	///	Where the herd is on the spiral around the player.
	float spiralIndex;
	///	The distance to the player (will only ever decrease).
	float playerDistance;

	///	True if the player's won the game.
	bool win;
	///	The point the herd rotates around when the player's won the game.
	TwoFloats winPoint;

	///	True if the player's picked up the homing beacon.
	bool homing;
};

#endif
