//	Elephant.cpp - A wee helper elephant.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Elephant.h"
#include "NiallsDrawingLib/Drawer.h"
#include "Box2DHelpers.h"
#include "Random.h"
#include "Pickup.h"

//------------------------------------------------------------------------------
Elephant::Elephant(b2World& world, const TwoFloats& playerPos)
{
	b2BodyDef bodyDef;
	b2FixtureDef fixtureDef;
	b2CircleShape boundingBox;
	TwoFloats pos;

	index = Random::getGlobal().getRandomFloat() * 359.99f;
	pos.x = sinf(index) * 256.0f;
	pos.y = -cosf(index) * 256.0f;
	pos += playerPos;

	bodyDef.type = b2_kinematicBody;
	bodyDef.position.Set(pixels2Meters(pos.x), pixels2Meters(pos.y));
	physicsBody = world.CreateBody(&bodyDef);

	boundingBox.m_p.Set(0.0f, 0.0f);
	boundingBox.m_radius = pixels2Meters(28.0f);
	fixtureDef.shape = &boundingBox;
	fixtureDef.density = 1.0f;
	fixtureDef.friction = 0.3f;
	fixtureDef.userData = this;
	physicsBody->CreateFixture(&fixtureDef);
}

//------------------------------------------------------------------------------
Elephant::~Elephant()
{

}

//------------------------------------------------------------------------------
void Elephant::move(const TwoFloats& playerPos)
{
	TwoFloats pos;

	pos.x = sinf(index) * 256.0f;
	pos.y = -cosf(index) * 256.0f;
	pos += playerPos;

	physicsBody->SetTransform(b2Vec2(pixels2Meters(pos.x), pixels2Meters(pos.y)), 0.0f);

	index += 0.025f;
	if(index > 6.283185f)
		index -= 6.283185f;
}

//------------------------------------------------------------------------------
void Elephant::draw(Drawer *d, const TwoFloats& playerPos)
{
	TwoFloats pos;

	pos.x = sinf(index) * 256.0f;
	pos.y = -cosf(index) * 256.0f;
	pos += playerPos;

	pos.x -= (playerPos.x - 640.0f);
	pos.y -= (playerPos.y - 360.0f);

	d->drawImage(Pickup::images[Pickup::Elephant],
				 pos.x-(8.0f*4.0f),
				 pos.y-(8.0f*4.0f),
				 4.0f,
				 4.0f);
}
