//	Elephant.h - A wee helper elephant.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef ELEPHANT_H_
#define ELEPHANT_H_

#include "Collidable.h"
#include "TwoFloats.h"

#include "Box2D/Box2D.h"
#include <stdint.h>

class Drawer;

///	A wee helper elephant.
class Elephant : public Collidable
{
  public:
	///	Constructor.
	Elephant(b2World& world, const TwoFloats& playerPos);
	///	Destructor.
	~Elephant();

	///	Moves the elephant.
	void move(const TwoFloats& playerPos);
	///	Draws the elephant.
	void draw(Drawer *d, const TwoFloats& playerPos);

	///	For collision purposes.
	const std::string getName() const {return "Elephant";};

	///	Returns the elephant's Box2D body.
	b2Body *getBody() const {return physicsBody;};
  private:
	///	The elephant's Box2D body.
	b2Body *physicsBody;

	///	The elephant's index in the circle it follows around the player.
	float index;
};

#endif
