//	FlameSpark.cpp - A spark from the donkey's bottom fire.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "FlameSpark.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "NiallsDrawingLib/OpenGLDrawer.h"
#include "Box2DHelpers.h"
#include "Images.h"
#include "Random.h"
#include "MiscHelper.h"

//------------------------------------------------------------------------------
uint32_t FlameSpark::image = 0;

//------------------------------------------------------------------------------
FlameSpark::FlameSpark(PlayerDonkey *player):
life(1.0f),
dead(false)
{
	int tempint;
	const ThreeFloats col[3] = {ThreeFloats(1.0f, 0.0f, 0.0f),
								ThreeFloats(1.0f, 0.5f, 0.0f),
								ThreeFloats(1.0f, 1.0f, 0.0f)};

	setPlayer(player);

	size = 4.0f + (Random::getGlobal().getRandomFloat() * 32.0f);

	tempint = (int)(Random::getGlobal().getRandomFloat() * 2.99f);
	colour = col[tempint];
}

//------------------------------------------------------------------------------
FlameSpark::~FlameSpark()
{
	
}

//------------------------------------------------------------------------------
void FlameSpark::initialise(b2World& world, const TwoFloats& pos, const TwoFloats& dir)
{
	float tempf;
	b2BodyDef bodyDef;
	b2FixtureDef fixtureDef;
	b2PolygonShape boundingBox;

	bodyDef.type = b2_dynamicBody;
	bodyDef.position.Set(pixels2Meters(pos.x), pixels2Meters(pos.y));
	physicsBody = world.CreateBody(&bodyDef);

	boundingBox.SetAsBox(pixels2Meters(size), pixels2Meters(size));
	fixtureDef.shape = &boundingBox;
	fixtureDef.density = 1.0f;
	fixtureDef.friction = 0.3f;
	fixtureDef.userData = this;
	fixtureDef.isSensor = true;
	physicsBody->CreateFixture(&fixtureDef);

	tempf = (Random::getGlobal().getRandomFloat() * 2.0f)-1.0f;
	if(size > 16.0f)
		tempf *= size * 0.0625f;
	else
		tempf *= 0.00001f;
	physicsBody->ApplyAngularImpulse(tempf);
	physicsBody->ApplyLinearImpulse(b2Vec2(pixels2Meters(dir.x*10.0f),
										   pixels2Meters(dir.y*10.0f)),
									physicsBody->GetWorldCenter());
}

//------------------------------------------------------------------------------
void FlameSpark::move()
{
	if(size > 16.0f)
		life -= 0.001f * size;
	else
		life -= 0.016f;
	if(life <= 0.0f)
		dead = true;
}

//------------------------------------------------------------------------------
void FlameSpark::draw(Drawer *d, const TwoFloats& playerPos)
{
	TwoFloats topLeft, bottomRight;
	TwoFloats pos(b2Vec22TwoFloats(physicsBody->GetPosition()));

	if(!image)
	{
		ImageData data;
		ImageLoader imLoad;
		OpenGLDrawer *openGLD = dynamic_cast<OpenGLDrawer *>(d);

		if(openGLD)
			openGLD->setTexInterp(false);

		data = imLoad.loadImage(Images::snowflake0_png, Images::snowflake0_pngSize);
		image = d->loadImage(data.data, data.width, data.height);

		if(openGLD)
			openGLD->setTexInterp(false);
	}

	topLeft.x = playerPos.x - 640.0f - (3.5f*size*0.5f);
	topLeft.y = playerPos.y - 360.0f - (3.5f*size*0.5f);
	bottomRight.x = playerPos.x + 640.0f + (3.5f*size*0.5f);
	bottomRight.y = playerPos.y + 360.0f + (3.5f*size*0.5f);

	pos.x = meters2Pixels(pos.x);
	pos.y = meters2Pixels(pos.y);
	
	if(pos.isWithin(topLeft, bottomRight))
	{
		topLeft += (3.5f*size*0.5f);

		d->drawImageRot(image,
						pos.x-topLeft.x - (3.5f*size),
						pos.y-topLeft.y - (3.5f*size),
						physicsBody->GetAngle() * (180.0f/3.14159265f),
						3.5f,
						3.5f,
						size,
						size,
						colour.x,
						colour.y,
						colour.z,
						fakeLog(life));
	}
}

//------------------------------------------------------------------------------
TwoFloats FlameSpark::getPosition() const
{
	TwoFloats retval;
	b2Vec2 physicsPos = physicsBody->GetPosition();

	retval.x = meters2Pixels(physicsPos.x);
	retval.y = meters2Pixels(physicsPos.y);

	return retval;
}

//------------------------------------------------------------------------------
void FlameSpark::setDead()
{
	dead = true;
}
