//	FlameSpark.h - A spark from the donkey's bottom fire.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef FlameSpark_H_
#define FlameSpark_H_

#include "PlayerDonkey.h"
#include "ThreeFloats.h"
#include "TwoFloats.h"

#include "Box2D/Box2D.h"

class Drawer;

///	A spark from the donkey's bottom fire.
class FlameSpark : public DonkeyFlames
{
  public:
	///	Constructor.
	FlameSpark(PlayerDonkey *player);
	///	Destructor.
	~FlameSpark();

	///	So we can delete the spark if it goes too far.
	void move();
	///	Initialises all the physics stuff.
	void initialise(b2World& world, const TwoFloats& pos, const TwoFloats& dir);

	///	Draws the FlameSpark.
	void draw(Drawer *d, const TwoFloats& playerPos);

	///	Returns the FlameSpark's position.
	TwoFloats getPosition() const;
	///	Returns the FlameSpark's Box2D body.
	b2Body *getBody() {return physicsBody;};

	///	Used to delete the FlameSpark.
	void setDead();
	///	Used to delete the FlameSpark.
	bool getDead() const {return dead;};

	///	Used in collisions.
	float getLife() const {return life;};
	///	Used in collisions.
	bool isSpark() const {return true;};
  private:
	///	The image we use (just a single pixel actually).
	static uint32_t image;

	///	The FlameSpark's Box2D body.
	b2Body *physicsBody;
	///	The size of the spark.
	float size;

	///	Used to delete the FlameSpark.
	float life;
	///	Used to delete the FlameSpark.
	bool dead;

	///	The spark's colour.
	ThreeFloats colour;
};

#endif
