/*
* Copyright (c) 2006-2007 Erin Catto http://www.gphysics.com
*
* iPhone port by Simon Oliver - http://www.simonoliver.com - http://www.handcircus.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#include "GLES-Render.h"
#include "PlayerDonkey.h"
#include "Box2DHelpers.h"


#include <cstdio>
#include <cstdarg>

#include <cstring>

GLESDebugDraw::GLESDebugDraw(PlayerDonkey *donkey):
player(donkey)
{

}

void GLESDebugDraw::DrawPolygon(const b2Vec2* vertices, int32 vertexCount, const b2Color& color)
{
	glTranslatef(player->getPosition().x-640.0f, player->getPosition().y-360.0f, 0.0f);

	glColor4f(color.r, color.g, color.b,1);
	glVertexPointer(2, GL_FLOAT, 0, vertices);
	glDrawArrays(GL_LINE_LOOP, 0, vertexCount);
}

void GLESDebugDraw::DrawSolidPolygon(const b2Vec2* vertices, int32 vertexCount, const b2Color& color)
{
	TwoFloats temp(player->getPosition().x-640.0f, player->getPosition().y-360.0f);
	b2Vec2 *newVertices = new b2Vec2[vertexCount];

	for(int i=0;i<vertexCount;++i)
	{
		newVertices[i].x = meters2Pixels(vertices[i].x) - temp.x;
		newVertices[i].y = meters2Pixels(vertices[i].y) - temp.y;
	}

	/*glPushMatrix();
	glTranslatef(temp.x, temp.y, 0.0f);
	glScalef(64.0f/2.0f, 64.0f/2.0f, 1.0f);*/

	glVertexPointer(2, GL_FLOAT, 0, newVertices);
	
	glColor4f(color.r, color.g, color.b,0.5f);
	glDrawArrays(GL_TRIANGLE_FAN, 0, vertexCount);
	
	glColor4f(color.r, color.g, color.b,1);
	glDrawArrays(GL_LINE_LOOP, 0, vertexCount);
	
	glColor4f(color.r, color.g, color.b,0.5f);
	glBegin(GL_QUADS);
	{
		glVertex2f(newVertices[0].x, newVertices[0].y);
		glVertex2f(newVertices[1].x, newVertices[1].y);
		glVertex2f(newVertices[2].x, newVertices[2].y);
		glVertex2f(newVertices[3].x, newVertices[3].y);
	}
	glEnd();
	glColor4f(color.r, color.g, color.b,1);
	glBegin(GL_LINE_LOOP);
	{
		glVertex2f(newVertices[0].x, newVertices[0].y);
		glVertex2f(newVertices[1].x, newVertices[1].y);
		glVertex2f(newVertices[2].x, newVertices[2].y);
		glVertex2f(newVertices[3].x, newVertices[3].y);
	}
	glEnd();
	
	//glPopMatrix();

	delete [] newVertices;
}

void GLESDebugDraw::DrawCircle(const b2Vec2& center, float32 radius, const b2Color& color)
{
	glTranslatef(player->getPosition().x-640.0f, player->getPosition().y-360.0f, 0.0f);

	const float32 k_segments = 16.0f;
	const int vertexCount=16;
	const float32 k_increment = 2.0f * b2_pi / k_segments;
	float32 theta = 0.0f;
	
	GLfloat glVertices[vertexCount*2];
	for (int32 i = 0; i < k_segments; ++i)
	{
		b2Vec2 v = center + radius * b2Vec2(cosf(theta), sinf(theta));
		glVertices[i*2]=v.x;
		glVertices[i*2+1]=v.y;
		theta += k_increment;
	}
	
	glColor4f(color.r, color.g, color.b,1);
	glVertexPointer(2, GL_FLOAT, 0, glVertices);
	
	glDrawArrays(GL_TRIANGLE_FAN, 0, vertexCount);
}

void GLESDebugDraw::DrawSolidCircle(const b2Vec2& center, float32 radius, const b2Vec2& axis, const b2Color& color)
{
	const float32 k_segments = 16.0f;
	const int vertexCount=16;
	const float32 k_increment = 2.0f * b2_pi / k_segments;
	float32 theta = 0.0f;
	b2Vec2 newCenter;
	TwoFloats temp(player->getPosition().x-640.0f, player->getPosition().y-360.0f);

	newCenter.x = meters2Pixels(center.x) - temp.x;
	newCenter.y = meters2Pixels(center.y) - temp.y;
	radius = meters2Pixels(radius);
	
	GLfloat glVertices[vertexCount*2];
	for (int32 i = 0; i < k_segments; ++i)
	{
		b2Vec2 v = newCenter + radius * b2Vec2(cosf(theta), sinf(theta));
		glVertices[i*2]=v.x;
		glVertices[i*2+1]=v.y;
		theta += k_increment;
	}
	
	glColor4f(color.r, color.g, color.b,0.5f);
	/*glVertexPointer(2, GL_FLOAT, 0, glVertices);
	glDrawArrays(GL_TRIANGLE_FAN, 0, vertexCount);*/
	glBegin(GL_TRIANGLE_FAN);
	{
		for(int i=0;i<vertexCount;++i)
			glVertex2f(glVertices[i*2], glVertices[(i*2)+1]);
	}
	glEnd();
	glColor4f(color.r, color.g, color.b,1);
	//glDrawArrays(GL_LINE_LOOP, 0, vertexCount);
	glBegin(GL_LINE_LOOP);
	{
		for(int i=0;i<vertexCount;++i)
			glVertex2f(glVertices[i*2], glVertices[(i*2)+1]);
	}
	glEnd();
	
	// Draw the axis line
	//DrawSegment(center,center+radius*axis,color);
}

void GLESDebugDraw::DrawSegment(const b2Vec2& p1, const b2Vec2& p2, const b2Color& color)
{
	TwoFloats temp(player->getPosition().x-640.0f, player->getPosition().y-360.0f);

	glColor4f(color.r, color.g, color.b,1);
	GLfloat				glVertices[] = {
		meters2Pixels(p1.x) - temp.x,
		meters2Pixels(p1.y) - temp.y,
		meters2Pixels(p2.x) - temp.x,
		meters2Pixels(p2.y) - temp.y
	};
	glVertexPointer(2, GL_FLOAT, 0, glVertices);
	glDrawArrays(GL_LINES, 0, 2);

	glBegin(GL_LINES);
	{
		glVertex2f(glVertices[0], glVertices[1]);
		glVertex2f(glVertices[2], glVertices[3]);
	}
	glEnd();
}

void GLESDebugDraw::DrawTransform(const b2Transform& xf)
{
	glTranslatef(player->getPosition().x-640.0f, player->getPosition().y-360.0f, 0.0f);

	b2Vec2 p1 = xf.p, p2;
	const float32 k_axisScale = 0.4f;

	p2 = p1 + k_axisScale * xf.q.GetXAxis();
	DrawSegment(p1,p2,b2Color(1,0,0));
	
	p2 = p1 + k_axisScale * xf.q.GetYAxis();
	DrawSegment(p1,p2,b2Color(0,1,0));
}

void GLESDebugDraw::DrawPoint(const b2Vec2& p, float32 size, const b2Color& color)
{
	glTranslatef(player->getPosition().x-640.0f, player->getPosition().y-360.0f, 0.0f);

	glColor4f(color.r, color.g, color.b,1);
	glPointSize(size);
	GLfloat				glVertices[] = {
		p.x,p.y
	};
	glVertexPointer(2, GL_FLOAT, 0, glVertices);
	glDrawArrays(GL_POINTS, 0, 1);
	glPointSize(1.0f);
}

void GLESDebugDraw::DrawString(int x, int y, const char *string, ...)
{

	/* Unsupported as yet. Could replace with bitmap font renderer at a later date */
}

void GLESDebugDraw::DrawAABB(b2AABB* aabb, const b2Color& c)
{
	glTranslatef(player->getPosition().x-640.0f, player->getPosition().y-360.0f, 0.0f);

	
	glColor4f(c.r, c.g, c.b,1);

	GLfloat				glVertices[] = {
		aabb->lowerBound.x, aabb->lowerBound.y,
		aabb->upperBound.x, aabb->lowerBound.y,
		aabb->upperBound.x, aabb->upperBound.y,
		aabb->lowerBound.x, aabb->upperBound.y
	};
	glVertexPointer(2, GL_FLOAT, 0, glVertices);
	glDrawArrays(GL_LINE_LOOP, 0, 8);
	
}
