//	GameOverScreen.cpp - The game over screen.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "GameOverScreen.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "NiallsDrawingLib/OpenGLDrawer.h"
#include "Images.h"
#include "Random.h"

//------------------------------------------------------------------------------
uint32_t GameOverScreen::backImages[3] = {0, 0, 0};
uint32_t GameOverScreen::ohNoesImage = 0;
uint32_t GameOverScreen::donkeyImage = 0;
uint32_t GameOverScreen::dedImage = 0;

//------------------------------------------------------------------------------
GameOverScreen::GameOverScreen(Application *app):
GameState(app),
colInterp(0.0f),
backCount(10),
backIndex(0),
startCount(30)
{
	int i;

	nextCol[0].x = 1.0f;
	updateCol();
	updateCol();

	for(i=0;i<32;++i)
	{
		TwoFloats pos;

		pos.x = Random::getGlobal().getRandomFloat() * 1280.0f;
		pos.y = Random::getGlobal().getRandomFloat() * 720.0f;
		stars.addStar(pos);
	}
	stars.setFade(false);
	stars.setSkulls(true);
}

//------------------------------------------------------------------------------
GameOverScreen::~GameOverScreen()
{

}

//------------------------------------------------------------------------------
void GameOverScreen::draw(Drawer *d)
{
	int i;

	if(!backImages[0])
	{
		ImageData data;
		ImageLoader imLoad;
		OpenGLDrawer *openGLD = dynamic_cast<OpenGLDrawer *>(d);

		data = imLoad.loadImage(Images::titleback0_png, Images::titleback0_pngSize);
		backImages[0] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::titleback1_png, Images::titleback1_pngSize);
		backImages[1] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::titleback2_png, Images::titleback2_pngSize);
		backImages[2] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::gameover_png, Images::gameover_pngSize);
		ohNoesImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::donkeyded_png, Images::donkeyded_pngSize);
		dedImage = d->loadImage(data.data, data.width, data.height);

		if(openGLD)
			openGLD->setTexInterp(false);

		data = imLoad.loadImage(Images::donkey_png, Images::donkey_pngSize);
		donkeyImage = d->loadImage(data.data, data.width, data.height);

		if(openGLD)
			openGLD->setTexInterp(true);
	}

	//Draw the background.
	for(i=0;i<3;++i)
	{
		ThreeFloats col;
		int index = (backIndex+i)%3;

		col = (1.0f-colInterp) * prevCol[index];
		col += colInterp * nextCol[index];

		d->drawImage(backImages[i], 0, 0, 1.0f, 1.0f, col.x, col.y, col.z);
		d->drawImageRot(backImages[i],
						1280.0f,
						720.0f,
						180.0f,
						0.0f,
						0.0f,
						1.0f,
						1.0f,
						col.x,
						col.y,
						col.z);
	}

	//Draw the stars.
	stars.draw(d, TwoFloats(640.0f, 360.0f));

	//Draw the oh noes! text.
	d->drawImage(ohNoesImage, 640.0f-256.0f, 32.0f);

	//Draw the donkey.
	d->drawImageRot(donkeyImage,
					640.0f-(4.0f*32.0f),
					360.0f-(4.0f*32.0f),
					270.0f,
					32.0f,
					32.0f,
					4.0f,
					4.0f);

	//Draw the donkey ded text.
	d->drawImage(dedImage, 640.0f-256.0f, 720.0f-32.0f-128.0f);

	colInterp += 0.005f;
	if(colInterp >= 1.0f)
	{
		colInterp = 0.0f;
		updateCol();
	}

	--backCount;
	if(backCount <= 0)
	{
		backCount = 10;
		++backIndex;
		backIndex %= 3;
	}

	if(startCount > 0)
		--startCount;
}

//------------------------------------------------------------------------------
void GameOverScreen::keyDown(const std::string& action, const SDL_keysym& keyPressed)
{
	if(startCount <= 0)
		nextState("TitleScreen");
}

//------------------------------------------------------------------------------
void GameOverScreen::updateCol()
{
	int col;
	ThreeFloats textCol;

	prevCol[0] = nextCol[0];
	prevCol[1] = nextCol[1];
	prevCol[2] = nextCol[2];

	//while(nextCol[0] == prevCol[0])
	{
		col = (int)(Random::getGlobal().getRandomFloat() * 7.99f);
		/*switch(col)
		{*/
			/*case 0: //Red
				nextCol[0].x = 1.0f;
				nextCol[0].y = 0.0f;
				nextCol[0].z = 0.0f;
				break;
			case 1: //Bright Green
				nextCol[0].x = 0.0f;
				nextCol[0].y = 1.0f;
				nextCol[0].z = 0.0f;
				break;
			case 2: //Blue
				nextCol[0].x = 0.0f;
				nextCol[0].y = 0.0f;
				nextCol[0].z = 1.0f;
				break;
			case 3: //Pink
				nextCol[0].x = 1.0f;
				nextCol[0].y = 0.0f;
				nextCol[0].z = 1.0f;
				break;
			case 4: //Yellow
				nextCol[0].x = 1.0f;
				nextCol[0].y = 1.0f;
				nextCol[0].z = 0.0f;
				break;
			case 5: //Orange
				nextCol[0].x = 1.0f;
				nextCol[0].y = 0.5f;
				nextCol[0].z = 0.0f;
				break;
			case 6: //Green
				nextCol[0].x = 0.0f;
				nextCol[0].y = 0.66f;
				nextCol[0].z = 0.0f;
				break;
			case 7: //Purple
				nextCol[0].x = 0.5f;
				nextCol[0].y = 0.0f;
				nextCol[0].z = 1.0f;
				break;*/
			//default: //Grey
				nextCol[0].x = 0.15f;
				nextCol[0].y = 0.15f;
				nextCol[0].z = 0.15f;
		//}
		nextCol[1] = nextCol[0] * 0.85f;
		nextCol[2] = nextCol[0] * 0.7f;
	}
}
