//	GameOverScreen.h - The game over screen.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef GAMEOVERSCREEN_H_
#define GAMEOVERSCREEN_H_

#include "GameState.h"
#include "ThreeFloats.h"
#include "StarParticles.h"

#include <stdint.h>

///	The title screen.
class GameOverScreen : public GameState
{
  public:
	///	Constructor.
	GameOverScreen(Application *app);
	///	Destructor.
	~GameOverScreen();

	///	Draws the state.
	void draw(Drawer *d);

	///	Called when a key down event is received.
	void keyDown(const std::string& action, const SDL_keysym& keyPressed);

	///	Returns the name of this state.
	const std::string getName() const {return "GameOverScreen";};
  private:
	///	Helper method. Updates prevCol and nextCol.
	void updateCol();

	///	The three background images.
	static uint32_t backImages[3];
	///	The oh noes text.
	static uint32_t ohNoesImage;
	///	The donkey image.
	static uint32_t donkeyImage;
	///	The donkey ded text.
	static uint32_t dedImage;

	///	Some star particles.
	StarParticles stars;

	///	The three colours used by the images.
	ThreeFloats prevCol[3];
	///	The three colours used by the images.
	ThreeFloats nextCol[3];
	///	Used to interpolate between the colours.
	float colInterp;
	///	Used to update backIndex.
	int backCount;
	///	Which colour corresponds to which background image.
	int backIndex;

	///	Used to make sure the player doesn't accidentally return to the title screen too soon.
	int startCount;
};

#endif
