//	GamePlay.cpp - Main gameplay state.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "GamePlay.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "NiallsDrawingLib/OpenGLDrawer.h"
#include "Santer.h"
#include "Random.h"
#include "MusicSingleton.h"
#include "Images.h"
#include "Pickup.h"
#include "Elephant.h"
#include "WreckingBall.h"
#include "MiscHelper.h"
#include "Box2DHelpers.h"
#include "Bomb.h"
#include "ShockWave.h"
#include "Nebula.h"
#include "LunarLander.h"
#include "Lazor.h"
#include "KingSanter.h"
#include "EmperorSanter.h"
#include "SanterSlay.h"
#include "Present.h"
#include "IceSanter.h"
#include "Application.h"
#include "Snowball.h"
#include "FlameSpark.h"
#include "Sounds.h"

#include <sstream>

using namespace std;

//------------------------------------------------------------------------------
uint32_t GamePlay::heartImage = 0;
uint32_t GamePlay::ohYeah = 0;
uint32_t GamePlay::backInTheHerd = 0;
uint32_t GamePlay::arrowImage = 0;
uint32_t GamePlay::pickupUI = 0;
uint32_t GamePlay::font = 0;

bool GamePlay::startInstruction = false;
bool GamePlay::santerInstruction = false;
bool GamePlay::nebulaInstruction = false;
bool GamePlay::kingInstruction = false;
bool GamePlay::slayInstruction = false;
bool GamePlay::iceInstruction = false;
bool GamePlay::winterInstruction = false;

//------------------------------------------------------------------------------
GamePlay::GamePlay(Application *app):
GameState(app),
physicsWorld(b2Vec2(0.0f, 0.0f)),
timeStep(1.0f/60.0f),
velocityIt(6),
positionIt(2),
santerCount(600),
santerToggle(8),
maxSanters(9),
emperor(false),
pickupCount(1200),
remainingPickups(4),
numActivatable(0),
activatablePickup(Pickup::NumTypes),
newActivatable(Pickup::NumTypes),
activateCooldown(0.0f),
instructionCount(600),
gameWin(false),
createJoint(0),
loudBit(false),
beat(0.0f),
addApple(true),
compass(false),
addElephant(false),
addWreckingBall(false),
stopTime(0.0),
pickupDisplay(0.0f),
lostCount(0.0f),
spawnedLostCompass(false),
titleCount(30),
debugDrawer(&player),
debugMode(false)
{
	unsigned int i, j;
	bool *startingPickups = getApplication()->getStartPickups();

	//Set up the physics stuff.
	physicsWorld.SetContactListener(this);
	player.initialise(physicsWorld);
	donkeyHerd.initialise(physicsWorld);
	debugDrawer.SetFlags(b2Draw::e_shapeBit|b2Draw::e_jointBit);
	physicsWorld.SetDebugDraw(&debugDrawer);

	vector<TwoFloats> tempVec;

	//Setup asteroids.
	for(j=0;j<8;++j)
	{
		float tempf;
		TwoFloats center;
		TwoFloats playerPos = player.getPosition();

		tempf = Random::getGlobal().getRandomFloat() * 1280.0f;
		center.x = playerPos.x + (sinf(((float)j/8.0f)*6.283185f) * (2000.0f+tempf));
		center.y = playerPos.y + (-cosf(((float)j/8.0f)*6.283185f) * (2000.0f+tempf));

		for(i=0;i<16;++i)
		{
			TwoFloats tempPos(center);

			tempPos.x += (Random::getGlobal().getRandomFloat() * 1000.0f)-500.0f;
			tempPos.y += (Random::getGlobal().getRandomFloat() * 600.0f)-300.0f;
			tempVec.push_back(tempPos);

			asteroids.push_back(StdSharedPtr<Asteroid>::type(new Asteroid()));
		}
	}
	for(i=0;i<asteroids.size();++i)
	{
		/*TwoFloats tempPos(center);

			tempPos.x += (Random::getGlobal().getRandomFloat() * 1000.0f)-500.0f;
			tempPos.y += (Random::getGlobal().getRandomFloat() * 600.0f)-300.0f;*/

		asteroids[i]->initialise(physicsWorld, tempVec[i]);
	}

	//Setup nebulae and lunar landers.
	for(i=0;i<8;++i)
	{
		float tempf;
		float tempf2;
		TwoFloats pos;
		TwoFloats playerPos = player.getPosition();

		tempf2 = ((float)i/8.0f);
		tempf2 += (Random::getGlobal().getRandomFloat() * 0.25f) - 0.125f;

		tempf = Random::getGlobal().getRandomFloat() * 2560.0f;

		pos.x = playerPos.x + (sinf(tempf2*6.283185f) * (3500.0f+tempf));
		pos.y = playerPos.y + (-cosf(tempf2*6.283185f) * (3500.0f+tempf));

		if(!(i%2))
		{
			lunarLanders.push_back(StdSharedPtr<LunarLander>::type(new LunarLander()));
			lunarLanders[i/2]->initialise(physicsWorld, pos);
		}
		else
		{
			nebulae.push_back(StdSharedPtr<Nebula>::type(new Nebula()));
			nebulae[i/2]->initialise(physicsWorld, pos);
		}
	}

	//Temp.
	/*santers.push_back(StdSharedPtr<Santer>::type(new SanterSlay()));
	santers[santers.size()-1]->initialise(physicsWorld, player.getPosition()+512.0f);*/

	if(!startInstruction)
	{
		instructionText = "OH NOES!\nRAINBOW SPACE DONKEY, Y U STRAY FROM HERD?\nDONT U NO MONSTER EVIL SPACE SANTERS ROAM DIS PART OV SPACE?";
		startInstruction = false;
	}

	if(getApplication()->getWinter())
		winterInstruction = true;

	pickupsMax[Pickup::Apple] = 10000;
	pickupsMax[Pickup::Chilli] = 1;
	pickupsMax[Pickup::Elephant] = 10;
	pickupsMax[Pickup::SpeedUp] = 10;
	pickupsMax[Pickup::Compass] = 1;
	pickupsMax[Pickup::MagnifyingGlass] = 1;
	pickupsMax[Pickup::Fire] = 1;
	pickupsMax[Pickup::WreckingBall] = 10;
	pickupsMax[Pickup::HomingBeacon] = 1;
	pickupsMax[Pickup::Buffer] = 1;
	pickupsMax[Pickup::Bomb] = 1;
	pickupsMax[Pickup::Turbo] = 1;
	pickupsMax[Pickup::Cake] = 1;
	pickupsMax[Pickup::Rock] = 1;
	pickupsMax[Pickup::Pigeon] = 1;
	pickupsMax[Pickup::Shield] = 1;
	pickupsMax[Pickup::UnicornHorn] = 1;

	if(getApplication()->getWinter())
	{
		pickupsMax[Pickup::IceCube] = 1;
		pickupsMax[Pickup::Scarf] = 1;
		pickupsMax[Pickup::Snowball] = 1;
		pickupsMax[Pickup::Stopwatch] = 1;
	}
	else
	{
		pickupsMax[Pickup::IceCube] = 0;
		pickupsMax[Pickup::Scarf] = 0;
		pickupsMax[Pickup::Snowball] = 0;
		pickupsMax[Pickup::Stopwatch] = 0;
	}

	//Add any starting pickups.
	for(i=0;i<(Pickup::NumTypes-1);++i)
	{
		if(startingPickups[i])
		{
			TwoFloats tempPos;
			TwoFloats playerPos = player.getPosition();

			tempPos.x = playerPos.x + (sinf(((float)i/(float)(Pickup::NumTypes-1))*6.283185f) * 300.0f);
			tempPos.y = playerPos.y + (-cosf(((float)i/(float)(Pickup::NumTypes-1))*6.283185f) * 300.0f);

			pickups.push_back(StdSharedPtr<Pickup>::type(new Pickup(i+1, tempPos, physicsWorld)));
		}
	}
}

//------------------------------------------------------------------------------
GamePlay::~GamePlay()
{
	
}

//------------------------------------------------------------------------------
void GamePlay::draw(Drawer *d)
{
	int i;
	TwoFloats playerPos;
	const bool winter = getApplication()->getWinter();

	if(!font)
	{
		ImageData data;
		ImageLoader imLoad;
		OpenGLDrawer *openGLD = dynamic_cast<OpenGLDrawer *>(d);

		data = imLoad.loadImage(Images::ohyeah_png, Images::ohyeah_pngSize);
		ohYeah = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::backintheherd_png, Images::backintheherd_pngSize);
		backInTheHerd = d->loadImage(data.data, data.width, data.height);

		if(openGLD)
			openGLD->setTexInterp(false);

		data = imLoad.loadImage(Images::heart_png, Images::heart_pngSize);
		heartImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::compassarrow_png, Images::compassarrow_pngSize);
		arrowImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupui_png, Images::pickupui_pngSize);
		pickupUI = d->loadImage(data.data, data.width, data.height);

		font = d->loadFont("MarsNeedsCunnilingus.ttf", 48.0f);

		if(openGLD)
			openGLD->setTexInterp(true);
	}

	if(createJoint)
	{
		//Attach player to donkey with a joint.
		b2RopeJointDef jointDef;

		jointDef.bodyA = player.getBody();
		jointDef.bodyB = createJoint->getBody();
		jointDef.localAnchorA = b2Vec2(0.0f, 1.0f);
		jointDef.localAnchorB = b2Vec2(0.0f, -1.0f);
		jointDef.maxLength = 16.0f;
		jointDef.collideConnected = true;

		physicsWorld.CreateJoint(&jointDef);
		donkeyHerd.gameWin(player.getPosition());

		createJoint = 0;
	}

	if(gameWin && (titleCount > 0))
		--titleCount;

	if((player.getLife() > 0.0f) && (!gameWin))
	{
		//Add new santers if necessary.
		--santerCount;
		if(loudBit) //Trigger more santers during the loud bit.
			--santerCount;
		if(santerCount < 0)
		{
			--santerToggle;
			if(santerToggle < 0)
			{
				santerCount = 800;
				santerToggle = maxSanters;
				++maxSanters;

				if(!emperor)
				{
					if(Random::getGlobal().getRandomFloat() > 0.75f)
					{
						santers.push_back(StdSharedPtr<Santer>::type(new EmperorSanter()));
						santers[santers.size()-1]->initialise(physicsWorld, getOutsideScreenPos());

						instructionText = "AN EMPEROR SANTER!\nAAAAAAAAAAAAAAAAAAAAIIIIIIIIIIIIIIIIIIIIIIIIIEEEEEEEEEEEEEEEEEEEEEE!!!!!!!!!!!!!!!!";
						instructionCount = 600;

						emperor = true;
					}
				}
				/*else if(loudBit)
				{
					if(Random::getGlobal().getRandomFloat() > 0.33f)
					{
						santers.push_back(StdSharedPtr<Santer>::type(new SanterSlay()));
						santers[santers.size()-1]->initialise(physicsWorld, getOutsideScreenPos());

						if(!slayInstruction)
						{
							instructionText = "ITS SANTER SLAY!\nWATCH OUT FOR HIS EXPLODING EVIL SPACE PRESENTS RAINBOW SPACE DONKEY!";
							instructionCount = 600;
							slayInstruction = true;
						}
					}
				}*/
			}
			else
			{
				TwoFloats pos = getOutsideScreenPos();

				//Last santer is a king.
				if(!santerToggle)
				{
					santers.push_back(StdSharedPtr<Santer>::type(new KingSanter()));

					if(!kingInstruction)
					{
						instructionText = "AAAAIIIEEE!\nA KING SANTER!\nWATCH YR BUM RAINBOW SPACE DONKEY!";
						instructionCount = 600;

						kingInstruction = true;
					}
				}
				///	In winter we get more ice santers.
				else if(winter && (santerToggle%3) && (Random::getGlobal().getRandomFloat() > 0.75f))
				//else if((santerToggle%3))
				{
					santers.push_back(StdSharedPtr<Santer>::type(new IceSanter()));

					if(!iceInstruction)
					{
						instructionText = "LOOK OUT, RAINBOW SPACE DONKEY!\nITS AN ICE SANTER!\nIF IT TOUCHES YOU YR BONES WILL FREEZ!";
						instructionCount = 600;

						iceInstruction = true;
					}
				}
				//5th santer might be an IceSanter.
				else if((santerToggle == 4) && (Random::getGlobal().getRandomFloat() > 0.9f))
				{
					santers.push_back(StdSharedPtr<Santer>::type(new IceSanter()));

					if(!iceInstruction)
					{
						instructionText = "LOOK OUT, RAINBOW SPACE DONKEY!\nITS AN ICE SANTER!\nIF IT TOUCHES YOU YR BONES WILL FREEZ!";
						instructionCount = 600;

						iceInstruction = true;
					}
				}
				//1st and 6th santers might be SanterSlay.
				else if(!(santerToggle%5) && loudBit && (Random::getGlobal().getRandomFloat() > 0.5f))
				{
					santers.push_back(StdSharedPtr<Santer>::type(new SanterSlay()));

					if(!slayInstruction)
					{
						instructionText = "ITS SANTER SLAY!\nWATCH OUT FOR HIS EXPLODING EVIL SPACE PRESENTS RAINBOW SPACE DONKEY!";
						instructionCount = 600;
						slayInstruction = true;
					}
				}
				else
					santers.push_back(StdSharedPtr<Santer>::type(new Santer()));
				santers[santers.size()-1]->initialise(physicsWorld, pos);
				santerCount = 200;

				if(!santerInstruction)
				{
					santerInstruction = true;
					instructionText = "OH NOES! OH NOES!\nA MONSTER EVIL SPACE SANTER!\nWATCH OUT, RAINBOW SPACE DONKEY!\nTRY AN BURN IT WITH YR FLAMY BOTTOM FIRE!";
					instructionCount = 600;
				}
			}
		}

		//Add any pickups.
		if(addApple)
		{
			TwoFloats pos;
			float tempf = Random::getGlobal().getRandomFloat() * 6.283185f;

			playerPos = player.getPosition();

			pos.x = playerPos.x + (sinf(tempf) * (640.0f+64.0f));
			pos.y = playerPos.y - (cosf(tempf) * (640.0f+64.0f));

			pickups.push_back(StdSharedPtr<Pickup>::type(new Pickup(Pickup::Apple, pos, physicsWorld)));

			addApple = false;
		}

		if(remainingPickups > 0)
		{
			--pickupCount;
			if(pickupCount < 0)
			{
				int tempint;
				TwoFloats pos;
				float tempf = Random::getGlobal().getRandomFloat() * 6.283185f;
				//int tempint = 1 + (int)(Random::getGlobal().getRandomFloat() * ((float)Pickup::UnicornHorn-0.01f));

				if(winter)
				{
					tempint = 1 + (int)(Random::getGlobal().getRandomFloat() *
										((float)Pickup::Stopwatch-0.01f));
				}
				else
				{
					tempint = 1 + (int)(Random::getGlobal().getRandomFloat() *
										((float)Pickup::UnicornHorn-0.01f));
				}

				playerPos = player.getPosition();

				pos.x = playerPos.x + (sinf(tempf) * (640.0f+64.0f));
				pos.y = playerPos.y - (cosf(tempf) * (640.0f+64.0f));

				//Ensure the player always gets one activatable pickup.
				if((remainingPickups == 1) && (numActivatable < 1))
				{
					if(winter)
					{
						tempint = Pickup::Buffer + (int)(Random::getGlobal().getRandomFloat() *
														 ((float)(Pickup::NumTypes-Pickup::HomingBeacon)-0.01f));
					}
					else
					{
						tempint = Pickup::Buffer + (int)(Random::getGlobal().getRandomFloat() *
														 ((float)(Pickup::Snowball-Pickup::HomingBeacon)-0.01f));
					}
				}
				else
				{
					while(pickupsMax[tempint] <= 0)
					{
						if(winter)
						{
							tempint = 1 + (int)(Random::getGlobal().getRandomFloat() *
												((float)Pickup::Stopwatch-0.01f));
						}
						else
						{
							tempint = 1 + (int)(Random::getGlobal().getRandomFloat() *
												((float)Pickup::UnicornHorn-0.01f));
						}
					}
				}

				//pickups.push_back(StdSharedPtr<Pickup>::type(new Pickup(Pickup::MagnifyingGlass, pos, physicsWorld)));
				pickups.push_back(StdSharedPtr<Pickup>::type(new Pickup(tempint, pos, physicsWorld)));
				--pickupsMax[tempint];

				if(tempint > Pickup::HomingBeacon)
					++numActivatable;

				pickupCount = 1000 + (int)(Random::getGlobal().getRandomFloat() * 400.0f);
				--remainingPickups;
			}
		}
		//Spawn a compass if it seems the player's lost.
		else if((lostCount > (3.0f * 60.0f)) &&
				!spawnedLostCompass &&
				(pickupsMax[Pickup::Compass] > 0))
		{
			TwoFloats pos;
			float tempf = Random::getGlobal().getRandomFloat() * 6.283185f;

			playerPos = player.getPosition();

			pos.x = playerPos.x + (sinf(tempf) * (640.0f+64.0f));
			pos.y = playerPos.y - (cosf(tempf) * (640.0f+64.0f));

			pickups.push_back(StdSharedPtr<Pickup>::type(new Pickup(Pickup::Compass, pos, physicsWorld)));

			spawnedLostCompass = true;
		}

		playerPos = player.getPosition();
		//Add any elephants.
		if(addElephant)
		{
			elephants.push_back(StdSharedPtr<Elephant>::type(new Elephant(physicsWorld, playerPos)));
			addElephant = false;
		}

		//Add any wrecking balls.
		if(addWreckingBall)
		{
			wreckingBalls.push_back(StdSharedPtr<WreckingBall>::type(new WreckingBall(physicsWorld,
																					  player.getBody(),
																					  playerPos)));
			addWreckingBall = false;
		}

		//Update any bombs.
		for(i=((int)bombs.size()-1);i>=0;--i)
		{
			if(bombs[i]->move())
			{
				bombExplosion(i);
				bombs.erase(bombs.begin()+i);

				Sounds::getInstance().playSound(Sounds::BombExplosion);
			}
		}
	}

	//Update all the world objects.
	if(!gameWin)
		player.move(this);
	playerPos = player.getPosition();
	if(stopTime <= 0.0f)
	{
		for(i=0;i<(int)santers.size();++i)
			santers[i]->move(playerPos, this);
		for(i=0;i<(int)pickups.size();++i)
			pickups[i]->move(playerPos);
		for(i=0;i<(int)elephants.size();++i)
			elephants[i]->move(playerPos);
		for(i=0;i<(int)lunarLanders.size();++i)
			lunarLanders[i]->move(this);
		for(i=((int)lazors.size()-1);i>=0;--i)
		{
			if(lazors[i]->move())
				lazors.erase(lazors.begin()+i);
		}
		for(i=0;i<(int)presents.size();++i)
			presents[i]->move();
		for(i=0;i<(int)snowballs.size();++i)
			snowballs[i]->move();
		for(i=0;i<(int)flameSparks.size();++i)
			flameSparks[i]->move();
		donkeyHerd.move(playerPos);
	}
	physicsWorld.Step(timeStep, (int)velocityIt, (int)positionIt);
	lazorCollisions();

	//Add a new pickup if the player's dropped one.
	if(newActivatable != Pickup::NumTypes)
	{
		if(activatablePickup != Pickup::NumTypes)
		{
			pickups.push_back(StdSharedPtr<Pickup>::type(new Pickup(activatablePickup,
																	player.getPosition(),
																	physicsWorld,
																	true)));
		}
		activatablePickup = newActivatable;
		activateCooldown = 0.0f;
		newActivatable = Pickup::NumTypes;
	}

	//Delete any dead santers.
	for(i=((int)santers.size()-1);i>=0;--i)
	{
		if(santers[i]->getDead())
		{
			if(santers[i]->getSanterType() != "Santer")
				shockWaves.push_back(ShockWave(santers[i]->getPosition()));
			if(santers[i]->getSanterType() == "IceSanter")
			{
				if(!getApplication()->getWinterUnlocked())
				{
					getApplication()->setWinter(true);

					if(!winterInstruction)
					{
						instructionText = "WINTER MODE UNLOCKED!";
						instructionCount = 600;

						winterInstruction = true;
					}
					Sounds::getInstance().playSound(Sounds::WinterModeUnlocked);
				}
			}

			santers[i]->takeDown();
			santers.erase(santers.begin()+i);
			Sounds::getInstance().playSound(Sounds::SanterDie);
		}
	}
	//Delete any dead pickups.
	for(i=((int)pickups.size()-1);i>=0;--i)
	{
		if(pickups[i]->isDead())
		{
			physicsWorld.DestroyBody(pickups[i]->getBody());
			pickups.erase(pickups.begin()+i);
		}
	}
	//Delete any dead presents.
	for(i=((int)presents.size()-1);i>=0;--i)
	{
		if(presents[i]->getDead())
		{
			physicsWorld.DestroyBody(presents[i]->getBody());
			presents.erase(presents.begin()+i);
		}
	}
	//Delete any dead snowballs.
	for(i=((int)snowballs.size()-1);i>=0;--i)
	{
		if(snowballs[i]->getDead())
		{
			physicsWorld.DestroyBody(snowballs[i]->getBody());
			snowballs.erase(snowballs.begin()+i);
		}
	}
	//Delete any dead flameSparks.
	for(i=((int)flameSparks.size()-1);i>=0;--i)
	{
		if(flameSparks[i]->getDead())
		{
			physicsWorld.DestroyBody(flameSparks[i]->getBody());
			flameSparks.erase(flameSparks.begin()+i);
		}
	}

	//Update the stopwatch time if necessary.
	if(stopTime > 0.0f)
		stopTime -= 0.015f;

	//Update the 'lost' counter.
	lostCount += (1.0f/60.0f); //Assume we're running at 60fps.

	//Draw the world.
	playerPos = player.getPosition();
	d->fillRect(0, 0, 1280, 360, 0, 0, 0, 255);
	{
		float red[3];
		float green[3];
		float blue[3];
		
		if(beat > 0.0f)
			beat -= 0.1f;
		if(beat < 0.0f)
			beat = 0.0f;

		red[0] = 32.0f;
		green[0] = 0.0f;
		blue[0] = 64.0f;

		red[1] = 255.0f;
		green[1] = 255.0f;
		blue[1] = 0.0f;

		red[2] = ((1.0f-beat)*red[0]) + (beat * red[1]);
		green[2] = ((1.0f-beat)*green[0]) + (beat * green[1]);
		blue[2] = ((1.0f-beat)*blue[0]) + (beat * blue[1]);
		d->fillGradient(0,
						360,
						1280,
						360,
						0,
						0,
						0,
						255,
						(uint8_t)(red[2]),
						(uint8_t)(green[2]),
						(uint8_t)(blue[2]),
						255);
	}

	stars.setScale(beat);
	stars.draw(d, playerPos);

	for(i=0;i<(int)pickups.size();++i)
		pickups[i]->draw(d, playerPos);
	for(i=0;i<(int)bombs.size();++i)
		bombs[i]->draw(d, playerPos);
	for(i=0;i<(int)santers.size();++i)
		santers[i]->draw(d, playerPos);
	for(i=0;i<(int)elephants.size();++i)
		elephants[i]->draw(d, playerPos);
	for(i=0;i<(int)wreckingBalls.size();++i)
		wreckingBalls[i]->draw(d, playerPos);
	for(i=0;i<(int)lunarLanders.size();++i)
		lunarLanders[i]->draw(d, playerPos);
	for(i=0;i<(int)presents.size();++i)
		presents[i]->draw(d, playerPos);
	for(i=0;i<(int)snowballs.size();++i)
		snowballs[i]->draw(d, playerPos);
	for(i=0;i<(int)flameSparks.size();++i)
		flameSparks[i]->draw(d, playerPos);

	player.draw(d);

	donkeyHerd.draw(d, playerPos);

	//Draw any asteroids.
	for(i=0;i<(int)asteroids.size();++i)
		asteroids[i]->draw(d, playerPos);

	//Draw any nebulae.
	for(i=0;i<(int)nebulae.size();++i)
		nebulae[i]->draw(d, playerPos, beat);

	//Draw any lazors.
	for(i=0;i<(int)lazors.size();++i)
		lazors[i]->draw(d, playerPos);

	//Draw/delete any explosions.
	for(i=((int)explosions.size()-1);i>=0;--i)
	{
		if(explosions[i].draw(d, playerPos))
			explosions.erase(explosions.begin()+i);
	}

	//Draw/delete any shock waves.
	for(i=((int)shockWaves.size()-1);i>=0;--i)
	{
		if(shockWaves[i].draw(d, playerPos))
			shockWaves.erase(shockWaves.begin()+i);
	}

	//Draw the compass arrow.
	if(compass)
	{
		float angle;
		TwoFloats pos;
		TwoFloats herdPos = donkeyHerd.getPosition();

		pos = (playerPos-herdPos).normal() * 192.0f;
		pos = 0.0f-pos;
		angle = atan2f(pos.y, pos.x);
		pos.x += 640.0f;
		pos.y += 360.0f;

		d->drawImageRot(arrowImage,
						pos.x - (3.5f * 4.0f),
						pos.y - (3.5f * 4.0f),
						(angle * (180.0f/3.14159265f))+90.0f,
						3.5f,
						3.5f,
						4.0f,
						4.0f);
	}

	//Draw pickup information.
	if(pickupDisplay > 0.0f)
	{
		float alpha = (pickupDisplay * 2.0f)-1.0f;

		if(alpha > 0.0f)
		{
			d->drawText(pickupName,
						font,
						(640.0f-128.0f),
						(360.0f-64.0f) - ((1.0f-pickupDisplay)*256.0f),
						512.0f,
						1.0f,
						1.0f,
						1.0f,
						1.0f,
						fakeLog(alpha));
		}
		d->drawText(pickupDesc,
					font,
					20.0f,
					(720.0f-64.0f),
					1024.0f);

		pickupDisplay -= 0.003f;
	}

	//Draw the donkey's life bar.
	if(!gameWin)
	{
		d->drawImage(heartImage, 16, 16, 4.0f, 4.0f);
		d->fillRect(96, 24, 384, 48, 255, 255, 255, 255);
		d->fillRect((96+4), (24+4), (384-8), (48-8), 0, 0, 0, 255);
		d->fillRect((96+10), (24+10), player.getLife() * (384-20), (48-20), 255, 0, 0, 255);

		d->drawImage(pickupUI, (1280.0f-(32*4.0f)), 16.0f, 4.0f, 4.0f);
		if(activatablePickup != Pickup::NumTypes)
		{
			float alpha = 1.0f;

			if(activateCooldown > 0.0f)
			{
				alpha = 0.25f;

				if(activatablePickup == Pickup::Snowball)
					activateCooldown -= 0.08f;
				else
					activateCooldown -= 0.001f;
			}
			d->drawImage(Pickup::images[activatablePickup],
						 (1280.0f-(32*4.0f)) + (8.0f*4.0f),
						 16.0f + (4.0f*4.0f),
						 4.0f,
						 4.0f,
						 1.0f,
						 1.0f,
						 1.0f,
						 alpha);
			if(activateCooldown > 0.0f)
			{
				d->fillRect((1280.0f-(32*4.0f)) + (8.0f*4.0f),
							16.0f + (4.0f*4.0f) + (14.0f*4.0f),
							(1.0f-activateCooldown) * (16.0f*4.0f),
							(2.0f * 4.0f),
							255,
							0,
							0,
							255);
			}
		}

		if(instructionCount > 0)
		{
			d->drawText(instructionText,
						font,
						100.0f,
						460.0f,
						1080.0f);
			--instructionCount;
		}
	}

	if(stopTime > 0.0f)
	{
		stringstream tempstr;

		tempstr << "Time frozen! " << (int)(stopTime + 1);

		d->drawText(tempstr.str(),
					font,
					800.0f,
					(720.0f-64.0f),
					1024.0f);
	}

	if(gameWin)
	{
		d->drawImage(ohYeah, 640.0f-256.0f, 32.0f);
		d->drawImage(backInTheHerd, 640.0f-512.0f, 720.0f-32.0f-128.0f);
	}

	//Temp.
	if(0)
	{
		float dist = donkeyHerd.getPosition().getDistance(playerPos);
		stringstream tempstr;

		tempstr << dist;

		d->drawText(tempstr.str(), font, 640.0f, 16.0f, 256.0f);
	}

	if(debugMode)
		physicsWorld.DrawDebugData();

	//Quit if the player's dead.
	if(!gameWin)
	{
		if(player.getLife() <= 0.0f)
		{
			Sounds::getInstance().playSound(Sounds::DonkeyDie);
			nextState("GameOverScreen");
		}
	}
}

//------------------------------------------------------------------------------
void GamePlay::keyDown(const string& action, const SDL_keysym& keyPressed)
{
	if(action == "Debug")
		debugMode = !debugMode;
	if(!gameWin)
	{
		if(action == "Escape")
			nextState("TitleScreen");
		else if((action == "P1Up")||(action == "P1Down")||(action == "P1Left")||(action == "P1Right"))
		{
			if(player.getLife() > 0.0f)
				player.keyDown(action);
		}

		if(action == "Action")
		{
			if((activatablePickup != Pickup::NumTypes) &&
			   (activateCooldown <= 0.0f))
			{
				activatePickup(activatablePickup);
				Sounds::getInstance().playSound(Sounds::PickupUsed);
			}
		}
	}
	else if(titleCount <= 0)
		nextState("TitleScreen");
}

//------------------------------------------------------------------------------
void GamePlay::keyUp(const string& action, const SDL_keysym& keyPressed)
{
	if((action == "P1Up")||(action == "P1Down")||(action == "P1Left")||(action == "P1Right"))
		player.keyUp(action);
}

//------------------------------------------------------------------------------
void GamePlay::BeginContact(b2Contact *contact)
{
	b2Fixture *fixtureA;
	b2Fixture *fixtureB;
	Collidable *collidableA = 0;
	Collidable *collidableB = 0;

	fixtureA = contact->GetFixtureA();
	fixtureB = contact->GetFixtureB();

	if(fixtureA->GetUserData())
		collidableA = (Collidable *)(fixtureA->GetUserData());
	if(fixtureB->GetUserData())
		collidableB = (Collidable *)(fixtureB->GetUserData());

	assert(collidableA);
	assert(collidableB);

	if(collidableA->getName() != collidableB->getName())
	{
		PlayerDonkey *player = 0;
		DonkeyFlames *flames = 0;
		Santer *santer = 0;
		Asteroid *asteroid = 0;
		DonkeyHerd::Donkey *donkey = 0;
		Pickup *pickup = 0;
		Elephant *elephant = 0;
		WreckingBall *wreckingBall = 0;
		Nebula *nebula = 0;
		LunarLander *lander = 0;
		Present *present = 0;
		Snowball *snowball = 0;

		if(collidableA->getName() == "PlayerDonkey")
			player = dynamic_cast<PlayerDonkey *>(collidableA);
		else if(collidableA->getName() == "DonkeyFlames")
			flames = dynamic_cast<DonkeyFlames *>(collidableA);
		else if(collidableA->getName() == "Santer")
			santer = dynamic_cast<Santer *>(collidableA);
		else if(collidableA->getName() == "Asteroid")
			asteroid = dynamic_cast<Asteroid *>(collidableA);
		else if(collidableA->getName() == "HerdDonkey")
			donkey = dynamic_cast<DonkeyHerd::Donkey *>(collidableA);
		else if(collidableA->getName() == "Pickup")
			pickup = dynamic_cast<Pickup *>(collidableA);
		else if(collidableA->getName() == "Elephant")
			elephant = dynamic_cast<Elephant *>(collidableA);
		else if(collidableA->getName() == "Nebula")
			nebula = dynamic_cast<Nebula *>(collidableA);
		else if(collidableA->getName() == "LunarLander")
			lander = dynamic_cast<LunarLander *>(collidableA);
		else if(collidableA->getName() == "Present")
			present = dynamic_cast<Present *>(collidableA);
		else if(collidableA->getName() == "Snowball")
			snowball = dynamic_cast<Snowball *>(collidableA);

		if(collidableB->getName() == "PlayerDonkey")
			player = dynamic_cast<PlayerDonkey *>(collidableB);
		else if(collidableB->getName() == "DonkeyFlames")
			flames = dynamic_cast<DonkeyFlames *>(collidableB);
		else if(collidableB->getName() == "Santer")
			santer = dynamic_cast<Santer *>(collidableB);
		else if(collidableB->getName() == "Asteroid")
			asteroid = dynamic_cast<Asteroid *>(collidableB);
		else if(collidableB->getName() == "HerdDonkey")
			donkey = dynamic_cast<DonkeyHerd::Donkey *>(collidableB);
		else if(collidableB->getName() == "Pickup")
			pickup = dynamic_cast<Pickup *>(collidableB);
		else if(collidableB->getName() == "Elephant")
			elephant = dynamic_cast<Elephant *>(collidableB);
		else if(collidableB->getName() == "WreckingBall")
			wreckingBall = dynamic_cast<WreckingBall *>(collidableB);
		else if(collidableB->getName() == "Nebula")
			nebula = dynamic_cast<Nebula *>(collidableB);
		else if(collidableB->getName() == "LunarLander")
			lander = dynamic_cast<LunarLander *>(collidableB);
		else if(collidableB->getName() == "Present")
			present = dynamic_cast<Present *>(collidableB);
		else if(collidableB->getName() == "Snowball")
			snowball = dynamic_cast<Snowball *>(collidableB);

		if(!asteroid)
		{
			if(santer && !flames && !player && !pickup && !nebula && !lander && !present)	//Santer's collided with the herd.
			{
				if(wreckingBall || elephant)
					santer->reduceHealth(2.0f);
				else if(snowball)
				{
					santer->reduceHealth(0.5f);
					explosions.push_back(SanterExplosion(santer->getPosition(), true));
				}
				else
					santer->reduceHealth(1000.0f);
				explosions.push_back(SanterExplosion(santer->getPosition()));
			}
			else if(santer && player)//Player's collided with a santer.
			{
				if(player->getInvincible())
				{
					santer->reduceHealth(1000.0f);
					explosions.push_back(SanterExplosion(santer->getPosition()));
				}
				else if(!player->getShield())
				{
					if(santer->getSanterType() == "KingSanter")
						player->updateLife(-0.1f);
					else
						player->updateLife(-0.02f);
					Sounds::getInstance().playSound(Sounds::SanterHitDonkey);

					if(santer->getSanterType() == "IceSanter")
						player->freeze();

					if(player->getIceCube())
						santer->freeze();
				}
			}
			else if(santer && flames) //Santer's collided with the player's flames.
			{
				if(flames->isSpark() && (flames->getLife() > 0.5f))
				{
					santer->reduceHealth(0.25f);
					explosions.push_back(SanterExplosion(santer->getPosition()));
				}
				else if(flames->getChilli() && this->player.getChilli())
				{
					santer->reduceHealth(1.0f);
					explosions.push_back(SanterExplosion(santer->getPosition()));
				}
				else if(!flames->getChilli() && this->player.isFlameOn())
				{
					santer->reduceHealth(1.0f);
					explosions.push_back(SanterExplosion(santer->getPosition()));
				}
			}
			else if(player && donkey)
			{
				if(!gameWin)
				{
					Sounds::getInstance().playSound(Sounds::FoundHerd);
					gameWin = true;
					createJoint = donkey;
					player->gameWin();
				}
			}
			else if(player && pickup)
			{
				if(pickup->getWait())
					collectPickup(pickup);
			}
			else if(present && player)
			{
				if(!player->getShield() && !player->getInvincible())
					player->updateLife(-0.1f);
				present->setDead();

				explosions.push_back(SanterExplosion(present->getPosition()));
			}
			else if(nebula)
			{
				if(player)
				{
					player->nebulaCollide(true);
					if(!nebulaInstruction)
					{
						instructionText = "OH NOES!\nA NEBULAS!\nIT'LL SUCK YOUR SPEED!\nBE CAREFUL RAINBOW SPACE DONKEY!";
						instructionCount = 600;
						nebulaInstruction = true;
					}
				}
				else if(santer)
					santer->nebulaCollide(true);
				else if(wreckingBall)
					wreckingBall->nebulaCollide(true);
				else if(lander)
					lander->nebulaCollide(true);
				else if(present)
					present->nebulaCollide(true);
			}
			else if(lander && (player || santer))
				lander->actorCollide();
		}
		else if(nebula)
			asteroid->nebulaCollide(true);
		else if(player || santer)
			Sounds::getInstance().playSound(Sounds::SanterHitDonkey);

		if(snowball && !player && !flames && !pickup && !nebula && !present)
			snowball->setDead();
	}
}

//------------------------------------------------------------------------------
void GamePlay::EndContact(b2Contact *contact)
{
	b2Fixture *fixtureA;
	b2Fixture *fixtureB;
	Collidable *collidableA = 0;
	Collidable *collidableB = 0;

	fixtureA = contact->GetFixtureA();
	fixtureB = contact->GetFixtureB();

	if(fixtureA->GetUserData())
		collidableA = (Collidable *)(fixtureA->GetUserData());
	if(fixtureB->GetUserData())
		collidableB = (Collidable *)(fixtureB->GetUserData());

	assert(collidableA);
	assert(collidableB);

	if(collidableA->getName() != collidableB->getName())
	{
		PlayerDonkey *player = 0;
		DonkeyFlames *flames = 0;
		Santer *santer = 0;
		Asteroid *asteroid = 0;
		DonkeyHerd::Donkey *donkey = 0;
		Pickup *pickup = 0;
		Elephant *elephant = 0;
		WreckingBall *wreckingBall = 0;
		Nebula *nebula = 0;
		LunarLander *lander = 0;
		Present *present = 0;

		if(collidableA->getName() == "PlayerDonkey")
			player = dynamic_cast<PlayerDonkey *>(collidableA);
		else if(collidableA->getName() == "DonkeyFlames")
			flames = dynamic_cast<DonkeyFlames *>(collidableA);
		else if(collidableA->getName() == "Santer")
			santer = dynamic_cast<Santer *>(collidableA);
		else if(collidableA->getName() == "Asteroid")
			asteroid = dynamic_cast<Asteroid *>(collidableA);
		else if(collidableA->getName() == "HerdDonkey")
			donkey = dynamic_cast<DonkeyHerd::Donkey *>(collidableA);
		else if(collidableA->getName() == "Pickup")
			pickup = dynamic_cast<Pickup *>(collidableA);
		else if(collidableA->getName() == "Elephant")
			elephant = dynamic_cast<Elephant *>(collidableA);
		else if(collidableA->getName() == "Nebula")
			nebula = dynamic_cast<Nebula *>(collidableA);
		else if(collidableA->getName() == "LunarLander")
			lander = dynamic_cast<LunarLander *>(collidableA);
		else if(collidableA->getName() == "Present")
			present = dynamic_cast<Present *>(collidableA);

		if(collidableB->getName() == "PlayerDonkey")
			player = dynamic_cast<PlayerDonkey *>(collidableB);
		else if(collidableB->getName() == "DonkeyFlames")
			flames = dynamic_cast<DonkeyFlames *>(collidableB);
		else if(collidableB->getName() == "Santer")
			santer = dynamic_cast<Santer *>(collidableB);
		else if(collidableB->getName() == "Asteroid")
			asteroid = dynamic_cast<Asteroid *>(collidableB);
		else if(collidableB->getName() == "HerdDonkey")
			donkey = dynamic_cast<DonkeyHerd::Donkey *>(collidableB);
		else if(collidableB->getName() == "Pickup")
			pickup = dynamic_cast<Pickup *>(collidableB);
		else if(collidableB->getName() == "Elephant")
			elephant = dynamic_cast<Elephant *>(collidableB);
		else if(collidableB->getName() == "WreckingBall")
			wreckingBall = dynamic_cast<WreckingBall *>(collidableB);
		else if(collidableB->getName() == "Nebula")
			nebula = dynamic_cast<Nebula *>(collidableB);
		else if(collidableB->getName() == "LunarLander")
			lander = dynamic_cast<LunarLander *>(collidableB);
		else if(collidableB->getName() == "Present")
			present = dynamic_cast<Present *>(collidableB);

		if(nebula)
		{
			if(player)
				player->nebulaCollide(false);
			else if(santer)
				santer->nebulaCollide(false);
			else if(asteroid)
				asteroid->nebulaCollide(false);
			else if(wreckingBall)
				wreckingBall->nebulaCollide(false);
			else if(lander)
				lander->nebulaCollide(false);
			else if(present)
				present->nebulaCollide(false);
		}
	}
}

//------------------------------------------------------------------------------
float32 GamePlay::ReportFixture(b2Fixture *fixture,
								const b2Vec2 &point,
								const b2Vec2 &normal,
								float32 fraction)
{
	float32 retval = 0.0f;
	Collidable *collidable = (Collidable *)(fixture->GetUserData());

	if(collidable)
	{
		PlayerDonkey *player = 0;
		Santer *santer = 0;
		Pickup *pickup = 0;

		if(collidable->getName() == "PlayerDonkey")
			player = dynamic_cast<PlayerDonkey *>(collidable);
		else if(collidable->getName() == "Santer")
			santer = dynamic_cast<Santer *>(collidable);
		else if(collidable->getName() == "Pickup")
			pickup = dynamic_cast<Pickup *>(collidable);

		if(pickup)
			retval = -1.0f;
		else if(player)
		{
			if(!player->getShield() && !player->getInvincible())
				player->updateLife(-0.1f);
			lazors[lazorIndex]->setDead();
		}
		else if(santer)
		{
			santer->reduceHealth(4.0f);
			lazors[lazorIndex]->setDead();
		}
	}

	return retval;
}

//------------------------------------------------------------------------------
void GamePlay::setLoudBit(bool val)
{
	loudBit = val;
	addApple = !addApple;
}

//------------------------------------------------------------------------------
void GamePlay::triggerBeat()
{
	beat = 1.0f;
}

//------------------------------------------------------------------------------
void GamePlay::addLazor(const TwoFloats& start, bool alsoTargetSanters)
{
	float dist;
	float tempf;
	TwoFloats dir;
	unsigned int i;
	int closestSanter = -1;

	dist = 100000.0f;
	if(alsoTargetSanters)
	{
		for(i=0;i<santers.size();++i)
		{
			tempf = santers[i]->getPosition().getDistance(start);
			if(tempf < dist)
			{
				dist = tempf;
				closestSanter = i;
			}
		}
	}

	tempf = player.getPosition().getDistance(start);
	if(tempf < dist)
	{
		dist = tempf;
		closestSanter = -1;
	}

	if(dist < 1280.0f) //Ignore targets too far away.
	{
		if(closestSanter > -1)
		{
			dir = (santers[closestSanter]->getPosition() - start).normal();

			lazors.push_back(StdSharedPtr<Lazor>::type(new Lazor(start, dir)));
			Sounds::getInstance().playSound(Sounds::Lazor);
		}
		else //Shoot at the player.
		{
			dir = (player.getPosition() - start).normal();

			lazors.push_back(StdSharedPtr<Lazor>::type(new Lazor(start, dir)));
			Sounds::getInstance().playSound(Sounds::Lazor);
		}
	}
}

//------------------------------------------------------------------------------
void GamePlay::addPresent(const TwoFloats& start)
{
	TwoFloats dir;
	b2Body *body;

	dir = (player.getPosition() - start).normal() * 3000.0f;

	presents.push_back(StdSharedPtr<Present>::type(new Present()));
	presents[presents.size()-1]->initialise(physicsWorld, start);
	body = presents[presents.size()-1]->getBody();
	body->ApplyLinearImpulse(b2Vec2(pixels2Meters(dir.x),
									pixels2Meters(dir.y)),
							 body->GetWorldCenter());
	Sounds::getInstance().playSound(Sounds::PresentFired);
}

//------------------------------------------------------------------------------
void GamePlay::addSpark(const TwoFloats& pos, const TwoFloats& dir)
{
	flameSparks.push_back(StdSharedPtr<FlameSpark>::type(new FlameSpark(&player)));
	flameSparks[flameSparks.size()-1]->initialise(physicsWorld, pos, dir);
}

//------------------------------------------------------------------------------
void GamePlay::bombExplosion(int index)
{
	float dest;
	b2Body *body;
	TwoFloats dir;
	TwoFloats pos;
	unsigned int i;
	TwoFloats bombPos = bombs[index]->getPosition();

	const float bombRadius = 512.0f;

	//Damage and move the player.
	pos = player.getPosition();
	dest = bombPos.getDistance(pos);
	if(dest < (bombRadius*0.5f))
	{
		player.updateLife(-0.33f);

		body = player.getBody();
		dir = (pos - bombPos).normal() * 1000.0f;
		body->ApplyLinearImpulse(twoFloats2B2Vec2(dir), body->GetWorldCenter());
	}

	//Kill any santers.
	for(i=0;i<santers.size();++i)
	{
		pos = santers[i]->getPosition();
		dest = bombPos.getDistance(pos);
		if(dest < bombRadius)
			santers[i]->reduceHealth(1000.0f);
	}

	//Move any asteroids.
	for(i=0;i<asteroids.size();++i)
	{
		pos = asteroids[i]->getPosition();
		dest = bombPos.getDistance(pos);
		if(dest < bombRadius)
		{
			body = asteroids[i]->getBody();
			dir = (pos - bombPos).normal() * 1000.0f;
			body->ApplyLinearImpulse(twoFloats2B2Vec2(dir), body->GetWorldCenter());
		}
	}

	//Add the explosion.
	explosions.push_back(SanterExplosion(bombPos));
	shockWaves.push_back(ShockWave(bombPos));
}

//------------------------------------------------------------------------------
void GamePlay::collectPickup(Pickup *pickup)
{
	pickup->setDead();

	switch(pickup->getType())
	{
		case Pickup::Apple:
			player.updateLife(0.25f);
			break;
		case Pickup::Chilli:
			player.triggerChilliFlames();
			break;
		case Pickup::SpeedUp:
			player.updateSpeed(2.0f);
			break;
		case Pickup::Compass:
			compass = true;
			break;
		case Pickup::MagnifyingGlass:
			player.setSmall();
			break;
		case Pickup::Fire:
			player.setConstantFire();
			break;
		case Pickup::HomingBeacon:
			player.setBeacon();
			donkeyHerd.setHoming();
			break;
		case Pickup::Elephant:
			addElephant = true;
			break;
		case Pickup::WreckingBall:
			addWreckingBall = true;
			break;
		case Pickup::IceCube:
			player.setIceCube();
			break;
		case Pickup::Scarf:
			player.setScarf();
			break;
		/*case Pickup::Cake:
			activatablePickup = Pickup::Cake;
			activateCooldown = 0.0f;
			break;
		case Pickup::Rock:
			activatablePickup = Pickup::Rock;
			activateCooldown = 0.0f;
			break;
		case Pickup::Buffer:
			activatablePickup = Pickup::Buffer;
			activateCooldown = 0.0f;
			break;
		case Pickup::Turbo:
			activatablePickup = Pickup::Turbo;
			activateCooldown = 0.0f;
			break;
		case Pickup::Pigeon:
			activatablePickup = Pickup::Pigeon;
			activateCooldown = 0.0f;
			break;
		case Pickup::UnicornHorn:
			activatablePickup = Pickup::UnicornHorn;
			activateCooldown = 0.0f;
			break;
		case Pickup::Shield:
			activatablePickup = Pickup::Shield;
			activateCooldown = 0.0f;
			break;
		case Pickup::Bomb:
			activatablePickup = Pickup::Bomb;
			activateCooldown = 0.0f;
			break;*/
		default:
			newActivatable = pickup->getType();
			break;
	}

	pickupDisplay = 1.0f;
	pickupName = pickup->getActualName();
	pickupDesc = pickup->getDescription();

	Sounds::getInstance().playSound(Sounds::PickupPickedUp);
}

//------------------------------------------------------------------------------
void GamePlay::activatePickup(Pickup::PickupType pickup)
{
	TwoFloats playerPos = player.getPosition();

	switch(pickup)
	{
		case Pickup::Buffer:
			{
				float dist;
				TwoFloats dir;
				unsigned int i;
				TwoFloats asteroidPos;
				b2Body *body;

				for(i=0;i<asteroids.size();++i)
				{
					asteroidPos = asteroids[i]->getPosition();
					dist = asteroidPos.getDistance(playerPos);

					if(dist < 448.0f)
					{
						body = asteroids[i]->getBody();
						dir = (asteroidPos - playerPos).normal() * 500.0f;
						body->ApplyLinearImpulse(twoFloats2B2Vec2(dir), body->GetWorldCenter());
					}
				}

				for(i=0;i<santers.size();++i)
				{
					asteroidPos = santers[i]->getPosition();
					dist = asteroidPos.getDistance(playerPos);

					if(dist < 448.0f)
					{
						body = santers[i]->getPhysicsBody();
						dir = (asteroidPos - playerPos).normal() * 500.0f;
						body->ApplyLinearImpulse(twoFloats2B2Vec2(dir), body->GetWorldCenter());
						santers[i]->setCooldown();
					}
				}
			}
			break;
		case Pickup::Bomb:
			bombs.push_back(StdSharedPtr<Bomb>::type(new Bomb(playerPos)));
			break;
		case Pickup::Turbo:
			{
				float dist;
				float angle;
				TwoFloats dir;
				unsigned int i;
				TwoFloats santerPos;
				b2Body *playerBody = player.getBody();

				angle = playerBody->GetAngle();
				dir.x = sinf(angle) * 10000.0f;
				dir.y = -cosf(angle) * 10000.0f;

				playerBody->ApplyLinearImpulse(twoFloats2B2Vec2(dir), playerBody->GetWorldCenter());

				//Make sure nearby santers don't chase us.
				for(i=0;i<santers.size();++i)
				{
					santerPos = santers[i]->getPosition();
					dist = santerPos.getDistance(playerPos);

					if(dist < 640.0f)
						santers[i]->setCooldown();
				}
			}
			break;
		case Pickup::Cake:
			player.updateLife(0.25f);
			break;
		case Pickup::Rock:
			{
				int i, j;
				TwoFloats pos;

				for(i=0,j=asteroids.size();i<8;++i,++j)
				{
					pos.x = playerPos.x + (sinf(((float)i/8.0f)*6.283185f) * 128.0f);
					pos.y = playerPos.y - (cosf(((float)i/8.0f)*6.283185f) * 128.0f);

					asteroids.push_back(StdSharedPtr<Asteroid>::type(new Asteroid()));
					asteroids[j]->initialise(physicsWorld, pos);
				}
			}
			break;
		case Pickup::Pigeon:
			{
				TwoFloats newPos = donkeyHerd.getPosition() - playerPos;

				newPos *= 0.5f;
				newPos += playerPos;
				player.teleport(newPos);
			}
			break;
		case Pickup::Shield:
			player.setShield();
			break;
		case Pickup::UnicornHorn:
			player.setInvincible();
			break;
		case Pickup::Snowball:
			{
				float angle;
				TwoFloats dir;

				angle = player.getBody()->GetAngle();
				dir.x = sinf(angle);
				dir.y = -cosf(angle);

				snowballs.push_back(StdSharedPtr<Snowball>::type(new Snowball()));
				snowballs[snowballs.size()-1]->initialise(physicsWorld,
														  player.getPosition(),
														  dir);
			}
			break;
		case Pickup::Stopwatch:
			{
				int i;

				for(i=0;i<(int)asteroids.size();++i)
					asteroids[i]->getBody()->SetAwake(false);
				for(i=0;i<(int)santers.size();++i)
					santers[i]->getPhysicsBody()->SetAwake(false);
				for(i=0;i<(int)pickups.size();++i)
					pickups[i]->getBody()->SetAwake(false);
				for(i=0;i<(int)elephants.size();++i)
					elephants[i]->getBody()->SetAwake(false);
				for(i=0;i<(int)lunarLanders.size();++i)
					lunarLanders[i]->getBody()->SetAwake(false);
				for(i=0;i<(int)presents.size();++i)
					presents[i]->getBody()->SetAwake(false);
				for(i=0;i<(int)snowballs.size();++i)
					snowballs[i]->getBody()->SetAwake(false);

				stopTime = 5.0f;
			}
			break;
	}
	activateCooldown = 1.0f;
}

//------------------------------------------------------------------------------
void GamePlay::lazorCollisions()
{
	unsigned int i;
	b2Vec2 start, end;

	for(i=0;i<lazors.size();++i)
	{
		start = twoFloats2B2Vec2(lazors[i]->getStart());
		start.x = pixels2Meters(start.x);
		start.y = pixels2Meters(start.y);

		end = twoFloats2B2Vec2(lazors[i]->getEnd());
		end.x = pixels2Meters(end.x);
		end.y = pixels2Meters(end.y);

		lazorIndex = i;
		physicsWorld.RayCast(this, start, end);
	}
}

//------------------------------------------------------------------------------
TwoFloats GamePlay::getOutsideScreenPos()
{
	TwoFloats retval;
	TwoFloats playerPos;
	int quadrant = (int)(Random::getGlobal().getRandomFloat() * 3.99f);

	playerPos = player.getPosition();
	switch(quadrant)
	{
		case 0: //Left.
			retval.x = Random::getGlobal().getRandomFloat() * 100.0f;
			retval.x += playerPos.x - 740.0f;

			retval.y = Random::getGlobal().getRandomFloat() * 920.0f;
			retval.y += playerPos.y - 460.0f;
			break;
		case 1: //Top.
			retval.x = Random::getGlobal().getRandomFloat() * 1480.0f;
			retval.x += playerPos.x - 740.0f;

			retval.y = Random::getGlobal().getRandomFloat() * 100.0f;
			retval.y += playerPos.y - 460.0f;
			break;
		case 2: //Right.
			retval.x = Random::getGlobal().getRandomFloat() * 100.0f;
			retval.x += playerPos.x + 640.0f;

			retval.y = Random::getGlobal().getRandomFloat() * 920.0f;
			retval.y += playerPos.y - 460.0f;
			break;
		case 3: //Bottom.
			retval.x = Random::getGlobal().getRandomFloat() * 1480.0f;
			retval.x += playerPos.x - 740.0f;

			retval.y = Random::getGlobal().getRandomFloat() * 100.0f;
			retval.y += playerPos.y + 360.0f;
			break;
	}

	return retval;
}
