//	IceSanter.h - An icy santer.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef ICESANTER_H_
#define ICESANTER_H_

#include "Santer.h"

///	A An icy santer.
class IceSanter : public Santer
{
  public:
	///	Constructor.
	IceSanter();
	///	Destructor.
	~IceSanter();

	///	Initialises the physics stuff.
	void initialise(b2World& world, const TwoFloats& pos);
	///	Takes down the physics stuff.
	void takeDown();

	///	Moves the IceSanter.
	void move(const TwoFloats& playerPos, GamePlay *gamePlay);
	///	Draws the IceSanter.
	void draw(Drawer *d, const TwoFloats& playerPos);

	///	Returns the IceSanter's position.
	TwoFloats getPosition() const;

	///	Used to kill the IceSanter.
	void reduceHealth(float val);
	///	Returns true if the player's killed the IceSanter.
	bool getDead() const {return dead;};
	///	Returns the IceSanter's Box2D body so GamePlay can destroy it.
	b2Body *getPhysicsBody() const {return physicsBody;};

	///	Used to ensure the IceSanter doesn't just bounce back when the player Buffers it.
	void setCooldown();

	///	Called when the IceSanter enters/leaves a nebula.
	void nebulaCollide(bool val);

	///	Used to slow the santer while it's frozen.
	void freeze();

	///	For collisions.
	const std::string getSanterType() const {return "IceSanter";};
  private:
	///	The IceSanter's body image.
	static uint32_t imageBody;
	///	The IceSanter's arm image.
	static uint32_t imageArm;

	///	The IceSanter's Box2D body.
	b2Body *physicsBody;
	///	The Box2D world instance so we can delete the physicsBody in the destructor.
	b2World *physicsWorld;

	///	The rotation of the IceSanter's arm.
	float armRot;

	///	True if the IceSanter should be killed.
	bool dead;

	///	Used to ensure the IceSanter doesn't just bounce back when the player Buffer's it.
	float coolDown;

	///	Used to make the santer occasionally charge the player.
	int chargeCount;
	
	///	True if the santer's inside a nebula.
	bool insideNebula;
	///	Used to freeze the santer if it's been hit by an IceCube-wielding donkey.
	float frozen;

	///	Temp.
	bool start;
};

#endif
