//	Lazor.cpp - A pew-pew lazor.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Lazor.h"
#include "NiallsDrawingLib/Drawer.h"

//------------------------------------------------------------------------------
Lazor::Lazor(const TwoFloats& start, const TwoFloats& dir):
startPos(start),
direction(dir),
length(72.0f),
speed(16.0f),
colourInterp(0.0f),
deathCount(1000),
dead(false)
{
	endPos = startPos + (direction * length);

	colours[0] = ThreeFloats(1.0f, 0.0f, 0.0f);
	colours[1] = ThreeFloats(1.0f, 0.5f, 0.0f);
	colours[2] = ThreeFloats(1.0f, 1.0f, 0.0f);
}

//------------------------------------------------------------------------------
Lazor::~Lazor()
{

}

//------------------------------------------------------------------------------
bool Lazor::move()
{
	bool retval = false;

	startPos += (direction * speed);
	endPos = startPos + (direction * length);

	--deathCount;
	if(deathCount <= 0)
		retval = true;

	if(dead)
		retval = true;

	return retval;
}

//------------------------------------------------------------------------------
void Lazor::draw(Drawer *d, const TwoFloats& playerPos)
{
	float tempf;
	int prev, next;
	ThreeFloats col;
	TwoFloats topLeft, bottomRight;

	prev = (int)colourInterp;
	next = (prev+1)%3;
	tempf = colourInterp - (float)prev;
	col = (1.0f-tempf) * colours[prev];
	col += tempf * colours[next];

	topLeft.x = playerPos.x - 640.0f;
	topLeft.y = playerPos.y - 360.0f;
	bottomRight.x = playerPos.x + 640.0f;
	bottomRight.y = playerPos.y + 360.0f;

	if(startPos.isWithin(topLeft, bottomRight) ||
	   endPos.isWithin(topLeft, bottomRight))
	{
		d->drawLine(startPos.x - topLeft.x,
					startPos.y - topLeft.y,
					endPos.x - topLeft.x,
					endPos.y - topLeft.y,
					col.x,
					col.y,
					col.z);
	}

	colourInterp += 0.33f;
	if(colourInterp >= 3.0f)
		colourInterp -= 3.0f;
}

//------------------------------------------------------------------------------
void Lazor::setDead()
{
	dead = true;
}
