//	Lazor.h - A pew-pew lazor.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef LAZOR_H_
#define LAZOR_H_

#include "ThreeFloats.h"
#include "TwoFloats.h"

class Drawer;

///	A pew-pew lazor.
class Lazor
{
  public:
	///	Constructor.
	Lazor(const TwoFloats& start, const TwoFloats& dir);
	///	Destructor.
	~Lazor();

	///	Moves the lazor.
	/*!
		\return True if the lazor should be deleted.
	 */
	bool move();
	///	Draws the lazor.
	void draw(Drawer *d, const TwoFloats& playerPos);

	///	Returns the current start point of the lazor.
	const TwoFloats& getStart() const {return startPos;};
	///	Returns the current end point of the lazor.
	const TwoFloats& getEnd() const {return endPos;};

	///	Tells the lazor it needs to be deleted.
	void setDead();
  private:
	///	The current start point of the lazor.
	TwoFloats startPos;
	///	The current end point of the lazor.
	TwoFloats endPos;
	///	The direction of the lazor.
	TwoFloats direction;
	///	The length of the lazor.
	const float length;
	///	The speed of the lazor.
	const float speed;

	///	Used to cycle the lazor's colour.
	float colourInterp;
	///	Used to cycle the lazor's colour.
	ThreeFloats colours[3];

	///	Used to ensure the lazor is deleted after a length of time.
	int deathCount;
	///	Used to kill the lazor when it collides with something.
	bool dead;
};

#endif
