//	LunarLander.h - A lunar lander that will attack if provoked.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef LUNARLANDER_H_
#define LUNARLANDER_H_

#include "Collidable.h"
#include "TwoFloats.h"

#include "Box2D/Box2D.h"
#include <stdint.h>

class Drawer;
class GamePlay;

///	A lunar lander that will attack if provoked.
class LunarLander : public Collidable
{
  public:
	///	Constructor.
	LunarLander();
	///	Destructor.
	~LunarLander();

	///	Initialises all the physics stuff.
	void initialise(b2World& world, const TwoFloats& pos);

	///	Moves the lander.
	void move(GamePlay *gamePlay);
	///	Draws the lander.
	void draw(Drawer *d, const TwoFloats& playerPos);

	///	Returns the LunarLander's position.
	TwoFloats getPosition() const;
	///	Returns the LunarLander's Box2D body.
	b2Body *getBody() {return physicsBody;};

	///	Called when the LunarLander enters/leaves a nebula.
	void nebulaCollide(bool val);
	///	Called when a santer or player collides with the lander.
	void actorCollide();

	///	For collisions.
	const std::string getName() const {return "LunarLander";};
  private:
	///	The lander's image.
	static uint32_t image;
	///	The 4 text images.
	static uint32_t text[4];

	///	The lander's Box2D body.
	b2Body *physicsBody;

	///	Used to move the lander in a slow orbit.
	float orbitIndex;
	///	The lander's speed.
	float orbitSpeed;

	///	Used to count how many times another actor's collided with the lander.
	int threeStrikes;
	///	Used to determine when to fire another laser.
	int laserCount;

	///	Used to time out the text.
	int textCount;
};

#endif
