//	MusicSingleton.cpp - Singleton class to switch between different tracks.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifdef DEBUG
#pragma warning(disable : 4786) //Stupid MS headers...
#endif

#include "MusicSingleton.h"
#include "BackgroundMusic.h"
#include "FilePath.h"
#include "Application.h"

using namespace std;

//------------------------------------------------------------------------------
MusicSingleton& MusicSingleton::getInstance()
{
	static MusicSingleton retval;

	return retval;
}

//------------------------------------------------------------------------------
void MusicSingleton::addTrack(const string& name)
{
	FilePath minusExtension(name);
	BackgroundMusic *music = new BackgroundMusic();

	music->setSoundFile(name);

	tracks.insert(make_pair(minusExtension.getName(), music));
	if(tracks.size() == 1)
	{
		tracks.begin()->second->start();
		currentTrack = tracks.begin()->first;
	}
}

//------------------------------------------------------------------------------
void MusicSingleton::playTrack(const std::string& name)
{
	map<string, BackgroundMusic *>::iterator it;

	if(name == currentTrack)
		return;

	it = tracks.find(name);
	if(it != tracks.end())
	{
		if(currentTrack != "triggers")
			tracks[currentTrack]->stop();
		tracks[name]->start();
		currentTrack = name;
	}
}

//------------------------------------------------------------------------------
void MusicSingleton::audioBlock(float *buffer, int numSamples)
{
	int i;

	if(!isMuted)
	{
		//First check if we've received any triggers.
		tracks["triggers"]->audioBlock(buffer, numSamples);
		for(i=0;i<numSamples;++i)
		{
			if(buffer[(i*2)] > 0.2f)
			{
				beats = true;
				beatCounter = 0.0f;

				application->setLoudBit(true);
				application->triggerBeat();
			}
			else if(buffer[(i*2)] < -0.2f)
			{
				application->setLoudBit(false);
				beats = false;
			}

			if(beats)
			{
				beatCounter += (1.0f/44100.0f);
				if(beatCounter > 0.5f)
				{
					beatCounter -= 0.5f;

					application->triggerBeat();
				}
			}
		}

		//Then clear the buffer.
		for(i=0;i<numSamples;++i)
		{
			buffer[(i*2)] = 0.0f;
			buffer[(i*2)+1] = 0.0f;
		}

		//Then play the actual music.
		tracks["RainbowSpaceDonkey"]->audioBlock(buffer, numSamples);
	}
}

//------------------------------------------------------------------------------
void MusicSingleton::mute()
{
	isMuted = !isMuted;
}

//------------------------------------------------------------------------------
void MusicSingleton::setApplication(Application *app)
{
	application = app;
}

//------------------------------------------------------------------------------
MusicSingleton::MusicSingleton():
isMuted(false),
application(0),
beatCounter(0.0f)
{
	addTrack("triggers.ogg");
	tracks["triggers"]->start();
}

//------------------------------------------------------------------------------
MusicSingleton::~MusicSingleton()
{
	map<string, BackgroundMusic *>::iterator it;

	for(it=tracks.begin();it!=tracks.end();++it)
		delete it->second;
}
