//	Nebula.cpp - A nebula to slow down actors.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Nebula.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "NiallsDrawingLib/OpenGLDrawer.h"
#include "Images.h"
#include "Box2DHelpers.h"
#include "ThreeFloats.h"

//------------------------------------------------------------------------------
uint32_t Nebula::image = 0;

//------------------------------------------------------------------------------
Nebula::Nebula():
physicsBody(0)
{

}

//------------------------------------------------------------------------------
Nebula::~Nebula()
{

}

//------------------------------------------------------------------------------
void Nebula::initialise(b2World& world, const TwoFloats& pos)
{
	b2BodyDef bodyDef;
	b2FixtureDef fixtureDef;
	b2CircleShape boundingBox;

	bodyDef.type = b2_dynamicBody;
	bodyDef.position.Set(pixels2Meters(pos.x), pixels2Meters(pos.y));
	physicsBody = world.CreateBody(&bodyDef);

	boundingBox.m_p.Set(0.0f, 0.0f);
	boundingBox.m_radius = pixels2Meters((64.0f-16.0f) * 12.0f);
	fixtureDef.shape = &boundingBox;
	fixtureDef.density = 1.0f;
	fixtureDef.friction = 0.3f;
	fixtureDef.userData = this;
	fixtureDef.isSensor = true;
	physicsBody->CreateFixture(&fixtureDef);

	physicsBody->SetTransform(bodyDef.position, 0.0f);
	physicsBody->SetLinearDamping(0.5f);
}

//------------------------------------------------------------------------------
void Nebula::draw(Drawer *d, const TwoFloats& playerPos, float beat)
{
	TwoFloats topLeft, bottomRight;
	TwoFloats pos(b2Vec22TwoFloats(physicsBody->GetPosition()));
	const ThreeFloats purple(1.0f, 0.0f, 0.5f);
	const ThreeFloats red(1.0f, 0.5f, 0.0f);
	ThreeFloats col;

	if(!image)
	{
		ImageData data;
		ImageLoader imLoad;
		OpenGLDrawer *openGLD = dynamic_cast<OpenGLDrawer *>(d);

		if(openGLD)
			openGLD->setTexInterp(false);

		data = imLoad.loadImage(Images::nebula_png, Images::nebula_pngSize);
		image = d->loadImage(data.data, data.width, data.height);

		if(openGLD)
			openGLD->setTexInterp(true);
	}

	topLeft.x = playerPos.x - 640.0f - (64.0f * 12.0f);
	topLeft.y = playerPos.y - 360.0f - (64.0f * 12.0f);
	bottomRight.x = playerPos.x + 640.0f + (64.0f * 12.0f);
	bottomRight.y = playerPos.y + 360.0f + (64.0f * 12.0f);

	pos.x = meters2Pixels(pos.x);
	pos.y = meters2Pixels(pos.y);

	col = (1.0f-beat) * purple;
	col += beat * red;
	
	if(pos.isWithin(topLeft, bottomRight))
	{
		topLeft += (64.0f * 10.0f);
		d->drawImage(image,
					 pos.x-topLeft.x-(64.0f*12.0f),
					 pos.y-topLeft.y-(64.0f*12.0f),
					 12.0f,
					 12.0f,
					 col.x,
					 col.y,
					 col.z);
	}
}
