//	Nebula.h - A nebula to slow down actors.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef NEBULA_H_
#define NEBULA_H_

#include "Collidable.h"
#include "TwoFloats.h"

#include "Box2D/Box2D.h"
#include <stdint.h>

class Drawer;

///	A nebula to slow down actors.
class Nebula : public Collidable
{
  public:
	///	Constructor.
	Nebula();
	///	Destructor.
	~Nebula();

	///	Initialises the nebula.
	void initialise(b2World& world, const TwoFloats& pos);

	///	Draws the nebula.
	void draw(Drawer *d, const TwoFloats& playerPos, float beat);

	///	For collisions.
	const std::string getName() const {return "Nebula";};
  private:
	///	The nebula's image.
	static uint32_t image;

	///	The nebula's Box2D body.
	b2Body *physicsBody;
};

#endif
