//	NewtonianActor.cpp - Base class for all actors using newtonian physics.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "NewtonianActor.h"

//------------------------------------------------------------------------------
NewtonianActor::NewtonianActor():
direction(0.0f, -1.0f),
fallOff(0.99f),
rotation(0.0f),
maxSpeed(16.0f)
{

}

//------------------------------------------------------------------------------
NewtonianActor::~NewtonianActor()
{

}

//------------------------------------------------------------------------------
void NewtonianActor::setPosition(const TwoFloats& pos)
{
	position = pos;
}

//------------------------------------------------------------------------------
void NewtonianActor::setDirection(const TwoFloats& dir)
{
	direction = dir;
}

//------------------------------------------------------------------------------
void NewtonianActor::update()
{
	const TwoFloats origin;
	TwoFloats lastPos = position;
	TwoFloats lastDir = direction;

	position += speed;

	if(lastPos != position)
	{
		direction = (position-lastPos).normal();

		if(direction.abs() != lastDir.abs())
		{
			lastDir = direction;
			lastDir.y = -lastDir.y;
			rotation = origin.getAngle(lastDir);
		}
	}

	if(speed.abs() > 0.0f)
	{
		speed *= fallOff;
		if(fabsf(speed.x) < 0.1f)
			speed.x = 0.0f;
		if(fabsf(speed.y) < 0.1f)
			speed.y = 0.0f;
	}
}

//------------------------------------------------------------------------------
void NewtonianActor::applyForce(const TwoFloats& force)
{
	speed += force;

	if(speed.x > maxSpeed)
		speed.x = maxSpeed;
	else if(speed.x < -maxSpeed)
		speed.x = -maxSpeed;

	if(speed.y > maxSpeed)
		speed.y = maxSpeed;
	else if(speed.y < -maxSpeed)
		speed.y = -maxSpeed;
}
