//	PickupSelectScreen.cpp - Bonus christmas pickup selection screen.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "PickupSelectScreen.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "NiallsDrawingLib/OpenGLDrawer.h"
#include "Images.h"
#include "Random.h"
#include "Application.h"

//------------------------------------------------------------------------------
uint32_t PickupSelectScreen::backImages[3] = {0, 0, 0};
uint32_t PickupSelectScreen::titleText = 0;
uint32_t PickupSelectScreen::mouseImage = 0;
uint32_t PickupSelectScreen::pickupUI = 0;

//------------------------------------------------------------------------------
PickupSelectScreen::PickupSelectScreen(Application *app):
GameState(app),
colInterp(0.0f),
backCount(10),
backIndex(0)
{
	int i;

	nextCol[0].x = 1.0f;
	updateCol();
	updateCol();
	updateCol();

	for(i=0;i<128;++i)
	{
		TwoFloats pos;

		pos.x = Random::getGlobal().getRandomFloat() * 1280.0f;
		pos.y = Random::getGlobal().getRandomFloat() * 720.0f;
		stars.addStar(pos);
	}
	stars.setFade(false);

	for(i=0;i<(Pickup::NumTypes-1);++i)
		selected[i] = false;
}

//------------------------------------------------------------------------------
PickupSelectScreen::~PickupSelectScreen()
{

}

//------------------------------------------------------------------------------
void PickupSelectScreen::draw(Drawer *d)
{
	int i;
	float x, y;
	const float pickupScale = 4.0f;

	if(!backImages[0])
	{
		ImageData data;
		ImageLoader imLoad;
		OpenGLDrawer *openGLD = dynamic_cast<OpenGLDrawer *>(d);

		data = imLoad.loadImage(Images::titleback0_png, Images::titleback0_pngSize);
		backImages[0] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::titleback1_png, Images::titleback1_pngSize);
		backImages[1] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::titleback2_png, Images::titleback2_pngSize);
		backImages[2] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupscreentext_png, Images::pickupscreentext_pngSize);
		titleText = d->loadImage(data.data, data.width, data.height);

		if(openGLD)
			openGLD->setTexInterp(false);

		data = imLoad.loadImage(Images::mousepointer_png, Images::mousepointer_pngSize);
		mouseImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupui2_png, Images::pickupui2_pngSize);
		pickupUI = d->loadImage(data.data, data.width, data.height);

		if(openGLD)
			openGLD->setTexInterp(true);

		Pickup::loadImages(d);
	}

	//Draw the background.
	for(i=0;i<3;++i)
	{
		ThreeFloats col;
		int index = (backIndex+i)%3;

		col = (1.0f-colInterp) * prevCol[index];
		col += colInterp * nextCol[index];

		d->drawImage(backImages[i], 0, 0, 1.0f, 1.0f, col.x, col.y, col.z);
		d->drawImageRot(backImages[i],
						1280.0f,
						720.0f,
						180.0f,
						0.0f,
						0.0f,
						1.0f,
						1.0f,
						col.x,
						col.y,
						col.z);
	}

	//Draw the stars.
	stars.draw(d, TwoFloats(640.0f, 360.0f));

	//Draw the title.
	d->drawImage(titleText, 640.0f-512.0f, 360.0f-512.0f);

	//Draw the pickups (passive).
	x = 64.0f + (8.0f * pickupScale);
	y = 320.0f;
	for(i=0;i<10;++i)
	{
		if(selected[i])
		{
			d->drawImage(pickupUI,
						 x - (16.0f * pickupScale),
						 y - (4.0f * pickupScale),
						 pickupScale,
						 pickupScale);
		}

		d->drawImage(Pickup::images[i+1],
					 x - (8.0f * pickupScale),
					 y,
					 pickupScale,
					 pickupScale);
		x += (1280.0f-64.0f)/10.0f;
	}

	//Draw the pickups (activatable).
	x = 64.0f + (8.0f * pickupScale);
	y = 440.0f;
	for(i=0;i<10;++i)
	{
		if(selected[i+10])
		{
			d->drawImage(pickupUI,
						 x - (16.0f * pickupScale),
						 y - (4.0f * pickupScale),
						 pickupScale,
						 pickupScale);
		}

		d->drawImage(Pickup::images[i+11],
					 x - (8.0f * pickupScale),
					 y,
					 pickupScale,
					 pickupScale);
		x += (1280.0f-64.0f)/10.0f;
	}

	//Draw the mouse pointer.
	d->drawImage(mouseImage, mousePos.x-4.0f, mousePos.y-4.0f, 4.0f, 4.0f);

	colInterp += 0.005f;
	if(colInterp >= 1.0f)
	{
		colInterp = 0.0f;
		updateCol();
	}

	--backCount;
	if(backCount <= 0)
	{
		backCount = 10;
		++backIndex;
		backIndex %= 3;
	}
}

//------------------------------------------------------------------------------
void PickupSelectScreen::keyDown(const std::string& action, const SDL_keysym& keyPressed)
{
	if(action == "Escape")
		nextState("TitleScreen");
	else
	{
		getApplication()->setStartPickups(selected);
		nextState("GamePlay");
	}
}

//------------------------------------------------------------------------------
void PickupSelectScreen::mouseMove(int posX, int posY)
{
	mousePos.x = (float)posX;
	mousePos.y = (float)posY;
}

//------------------------------------------------------------------------------
void PickupSelectScreen::mouseDown(int button, int posX, int posY)
{
	int i;
	float x, y;
	TwoFloats topLeft, bottomRight;
	const float pickupScale = 4.0f;
	TwoFloats mouse = TwoFloats((float)posX, (float)posY);

	//Passive pickups.
	x = 64.0f + (8.0f * pickupScale);
	y = 320.0f;
	for(i=0;i<10;++i)
	{
		topLeft.x = x - (8.0f*pickupScale);
		topLeft.y = y;
		bottomRight = topLeft + (16.0f*pickupScale);

		if(mouse.isWithin(topLeft, bottomRight))
		{
			selected[i] = !selected[i];
			break;
		}

		x += (1280.0f-64.0f)/10.0f;
	}

	//Activatable pickups.
	x = 64.0f + (8.0f * pickupScale);
	y = 440.0f;
	for(i=0;i<10;++i)
	{
		topLeft.x = x - (8.0f*pickupScale);
		topLeft.y = y;
		bottomRight = topLeft + (16.0f*pickupScale);

		if(mouse.isWithin(topLeft, bottomRight))
		{
			selected[i+10] = !selected[i+10];
			break;
		}

		x += (1280.0f-64.0f)/10.0f;
	}
}

//------------------------------------------------------------------------------
void PickupSelectScreen::updateCol()
{
	int col;
	ThreeFloats textCol;

	prevCol[0] = nextCol[0];
	prevCol[1] = nextCol[1];
	prevCol[2] = nextCol[2];

	while(nextCol[0] == prevCol[0])
	{
		col = (int)(Random::getGlobal().getRandomFloat() * 7.99f);
		switch(col)
		{
			case 0: //Red
				nextCol[0].x = 1.0f;
				nextCol[0].y = 0.0f;
				nextCol[0].z = 0.0f;
				break;
			case 1: //Bright Green
				nextCol[0].x = 0.0f;
				nextCol[0].y = 1.0f;
				nextCol[0].z = 0.0f;
				break;
			case 2: //Blue
				nextCol[0].x = 0.0f;
				nextCol[0].y = 0.0f;
				nextCol[0].z = 1.0f;
				break;
			case 3: //Pink
				nextCol[0].x = 1.0f;
				nextCol[0].y = 0.0f;
				nextCol[0].z = 1.0f;
				break;
			case 4: //Yellow
				nextCol[0].x = 1.0f;
				nextCol[0].y = 1.0f;
				nextCol[0].z = 0.0f;
				break;
			case 5: //Orange
				nextCol[0].x = 1.0f;
				nextCol[0].y = 0.5f;
				nextCol[0].z = 0.0f;
				break;
			case 6: //Green
				nextCol[0].x = 0.0f;
				nextCol[0].y = 0.66f;
				nextCol[0].z = 0.0f;
				break;
			case 7: //Purple
				nextCol[0].x = 0.5f;
				nextCol[0].y = 0.0f;
				nextCol[0].z = 1.0f;
				break;
		}
		nextCol[1] = nextCol[0] * 0.85f;
		nextCol[2] = nextCol[0] * 0.7f;
	}
}
