//	PickupSelectScreen.h - Bonus christmas pickup selection screen.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef PICKUPSELECTSCREEN_H_
#define PICKUPSELECTSCREEN_H_

#include "GameState.h"
#include "ThreeFloats.h"
#include "StarParticles.h"
#include "Pickup.h"

#include <stdint.h>

///	The title screen.
class PickupSelectScreen : public GameState
{
  public:
	///	Constructor.
	PickupSelectScreen(Application *app);
	///	Destructor.
	~PickupSelectScreen();

	///	Draws the state.
	void draw(Drawer *d);

	///	Called when a key down event is received.
	void keyDown(const std::string& action, const SDL_keysym& keyPressed);
	///	Called when a mouse move event is received.
	void mouseMove(int posX, int posY);
	///	Called when a mouse down event is received.
	void mouseDown(int button, int posX, int posY);

	///	Returns the name of this state.
	const std::string getName() const {return "PickupSelectScreen";};
  private:
	///	Helper method. Updates prevCol and nextCol.
	void updateCol();

	///	The three background images.
	static uint32_t backImages[3];
	///	The title text image.
	static uint32_t titleText;
	///	The mouse pointer image.
	static uint32_t mouseImage;
	///	The pickup UI image.
	static uint32_t pickupUI;

	///	Some star particles.
	StarParticles stars;

	///	The three colours used by the images.
	ThreeFloats prevCol[3];
	///	The three colours used by the images.
	ThreeFloats nextCol[3];
	///	Used to interpolate between the colours.
	float colInterp;
	///	Used to update backIndex.
	int backCount;
	///	Which colour corresponds to which background image.
	int backIndex;

	///	The mouse pointer position.
	TwoFloats mousePos;

	///	Which pickups are selected.
	bool selected[Pickup::NumTypes-1];
};

#endif
