//	Present.cpp - A present thrown by a SanterSlay.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Present.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "NiallsDrawingLib/OpenGLDrawer.h"
#include "Images.h"
#include "Box2DHelpers.h"
#include "Random.h"

//------------------------------------------------------------------------------
uint32_t Present::images[7] = {0, 0, 0, 0, 0, 0, 0};

//------------------------------------------------------------------------------
Present::Present():
deathCount(2000),
dead(false)
{
	
}

//------------------------------------------------------------------------------
Present::~Present()
{
	
}

//------------------------------------------------------------------------------
void Present::initialise(b2World& world, const TwoFloats& pos)
{
	b2BodyDef bodyDef;
	b2FixtureDef fixtureDef;
	b2CircleShape boundingBox;

	bodyDef.type = b2_dynamicBody;
	bodyDef.position.Set(pixels2Meters(pos.x), pixels2Meters(pos.y));
	physicsBody = world.CreateBody(&bodyDef);

	boundingBox.m_p.Set(0.0f, 0.0f);
	boundingBox.m_radius = pixels2Meters((8.0f-2.0f)*6.0f);
	fixtureDef.shape = &boundingBox;
	fixtureDef.density = 1.0f;
	fixtureDef.friction = 0.3f;
	fixtureDef.userData = this;
	fixtureDef.isSensor = true;
	physicsBody->CreateFixture(&fixtureDef);

	//physicsBody->SetTransform(bodyDef.position, 0.0f);
	//physicsBody->SetLinearDamping(0.5f);

	imageIndex = (int)(Random::getGlobal().getRandomFloat() * 6.99f);
}

//------------------------------------------------------------------------------
void Present::move()
{
	--deathCount;
	if(deathCount <= 0)
		dead = true;
}

//------------------------------------------------------------------------------
void Present::draw(Drawer *d, const TwoFloats& playerPos)
{
	TwoFloats topLeft, bottomRight;
	TwoFloats pos(b2Vec22TwoFloats(physicsBody->GetPosition()));

	if(!images[0])
	{
		ImageData data;
		ImageLoader imLoad;
		OpenGLDrawer *openGLD = dynamic_cast<OpenGLDrawer *>(d);

		if(openGLD)
			openGLD->setTexInterp(false);

		data = imLoad.loadImage(Images::santermess1_png, Images::santermess1_pngSize);
		images[0] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::present1_png, Images::present1_pngSize);
		images[1] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::present2_png, Images::present2_pngSize);
		images[2] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::present3_png, Images::present3_pngSize);
		images[3] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::present4_png, Images::present4_pngSize);
		images[4] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::present5_png, Images::present5_pngSize);
		images[5] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::present6_png, Images::present6_pngSize);
		images[6] = d->loadImage(data.data, data.width, data.height);

		if(openGLD)
			openGLD->setTexInterp(true);
	}

	topLeft.x = playerPos.x - 640.0f - (8.0f*6.0f);
	topLeft.y = playerPos.y - 360.0f - (8.0f*6.0f);
	bottomRight.x = playerPos.x + 640.0f + (8.0f*6.0f);
	bottomRight.y = playerPos.y + 360.0f + (8.0f*6.0f);

	pos.x = meters2Pixels(pos.x);
	pos.y = meters2Pixels(pos.y);
	
	if(pos.isWithin(topLeft, bottomRight))
	{
		topLeft += (8.0f*6.0f);
		d->drawImageRot(images[imageIndex],
						pos.x-topLeft.x - (8.0f*6.0f),
						pos.y-topLeft.y - (8.0f*6.0f),
						physicsBody->GetAngle() * (180.0f/3.14159265f),
						8.0f,
						8.0f,
						6.0f,
						6.0f);
	}
}

//------------------------------------------------------------------------------
TwoFloats Present::getPosition() const
{
	TwoFloats retval;
	b2Vec2 physicsPos = physicsBody->GetPosition();

	retval.x = meters2Pixels(physicsPos.x);
	retval.y = meters2Pixels(physicsPos.y);

	return retval;
}

//------------------------------------------------------------------------------
void Present::nebulaCollide(bool val)
{
	if(val)
	{
		physicsBody->SetLinearDamping(2.0f);
		physicsBody->SetAngularDamping(2.0f);
	}
	else
	{
		physicsBody->SetLinearDamping(0.5f);
		physicsBody->SetAngularDamping(0.05f);
	}
}

//------------------------------------------------------------------------------
void Present::setDead()
{
	dead = true;
}
