//	Present.h - A present thrown by a SanterSlay.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef PRESENT_H_
#define PRESENT_H_

#include "Collidable.h"
#include "TwoFloats.h"

#include "Box2D/Box2D.h"
#include <stdint.h>

class Drawer;

///	A present thrown by a SanterSlay.
class Present : public Collidable
{
  public:
	///	Constructor.
	Present();
	///	Destructor.
	~Present();

	///	Initialises all the physics stuff.
	void initialise(b2World& world, const TwoFloats& pos);

	///	Used to ensure the present doesn't last forever if it doesn't hit the player.
	void move();
	///	Draws the Present.
	void draw(Drawer *d, const TwoFloats& playerPos);

	///	Returns the Present's position.
	TwoFloats getPosition() const;
	///	Returns the Present's Box2D body.
	b2Body *getBody() {return physicsBody;};

	///	Called when the Present enters/leaves a nebula.
	void nebulaCollide(bool val);

	///	Used to delete the present.
	void setDead();
	///	Used to delete the present.
	bool getDead() const {return dead;};

	///	For collisions.
	const std::string getName() const {return "Present";};
  private:
	///	The 7 Present images.
	static uint32_t images[7];

	///	Which image this Present uses.
	int imageIndex;

	///	The Present's Box2D body.
	b2Body *physicsBody;

	///	Used to delete the present.
	int deathCount;
	///	Used to delete the present.
	bool dead;
};

#endif
