//	Santer.h - A monster evil space santer.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef SANTER_H_
#define SANTER_H_

#include "Collidable.h"
#include "TwoFloats.h"

#include "Box2D/Box2D.h"
#include <stdint.h>

class Drawer;
class GamePlay;

///	A monster evil space santer.
/*!
	Also base class for other santer types. Yes it's ugly and clunky, but I
	started this way, and I can't be bothered fixing it properly.
 */
class Santer : public Collidable
{
  public:
	///	Constructor.
	Santer();
	///	Destructor.
	virtual ~Santer();

	///	Initialises the physics stuff.
	virtual void initialise(b2World& world, const TwoFloats& pos);
	///	Takes down the physics stuff.
	virtual void takeDown();

	///	Moves the santer.
	virtual void move(const TwoFloats& playerPos, GamePlay *gamePlay);
	///	Draws the santer.
	virtual void draw(Drawer *d, const TwoFloats& playerPos);

	///	Returns the santer's position.
	virtual TwoFloats getPosition() const;

	///	Used to kill the santer.
	virtual void reduceHealth(float val);
	///	Returns true if the player's killed the santer.
	virtual bool getDead() const {return (life <= 0.0f);};
	///	Returns the santer's Box2D body so GamePlay can destroy it.
	virtual b2Body *getPhysicsBody() const {return physicsBody;};

	///	Used to ensure the santer doesn't just bounce back when the player Buffers it.
	virtual void setCooldown();

	///	Called when the santer enters/leaves a nebula.
	virtual void nebulaCollide(bool val);

	///	Used to slow the santer while it's frozen.
	virtual void freeze();

	///	For collisions.
	const std::string getName() const {return "Santer";};
	///	For collisions.
	virtual const std::string getSanterType() const {return "Santer";};
  protected:
	///	The santer's body image.
	static uint32_t imageBody;
	///	The santer's arm image.
	static uint32_t imageArm;
  private:
	///	The santer's Box2D body.
	b2Body *physicsBody;
	///	The Box2D world instance so we can delete the physicsBody in the destructor.
	b2World *physicsWorld;

	///	The rotation of the santer's arm.
	float armRot;

	///	True if the santer should be killed.
	float life;

	///	Used to ensure the santer doesn't just bounce back when the player Buffers it.
	float coolDown;
	
	///	True if the santer's inside a nebula.
	bool insideNebula;
	///	Used to freeze the santer if it's been hit by an IceCube-wielding donkey.
	float frozen;

	///	Temp.
	bool start;
};

#endif
