//	SanterExplosion.cpp - Class representing the explosion of a Santer.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "SanterExplosion.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "NiallsDrawingLib/OpenGLDrawer.h"
#include "Images.h"
#include "Random.h"

//------------------------------------------------------------------------------
uint32_t SanterExplosion::images[6] = {0, 0, 0, 0, 0, 0};
uint32_t SanterExplosion::snowflakeImages[4] = {0, 0, 0, 0};

//------------------------------------------------------------------------------
SanterExplosion::SanterExplosion(const TwoFloats& pos, bool snowflakeExplosion):
position(pos),
numDead(0),
snowflake(snowflakeExplosion)
{
	int i;

	for(i=0;i<NumParticles;++i)
	{
		particles[i].pos = position;
		while((particles[i].dir.x == 0.0f) && (particles[i].dir.y == 0.0f))
		{
			particles[i].dir.x = (Random::getGlobal().getRandomFloat()*2.0f) - 1.0f;
			particles[i].dir.y = (Random::getGlobal().getRandomFloat()*2.0f) - 1.0f;
		}
		particles[i].dir = particles[i].dir.normal();
		particles[i].speed = 1.0f + (Random::getGlobal().getRandomFloat() * 5.0f);
		particles[i].angle = Random::getGlobal().getRandomFloat() * 359.0f;
		particles[i].rotSpeed = 1.0f + (Random::getGlobal().getRandomFloat() * 2.0f);
		if(Random::getGlobal().getRandomFloat() > 0.5f)
			particles[i].rotSpeed = -particles[i].rotSpeed;
		if(snowflake)
			particles[i].image = (int)(Random::getGlobal().getRandomFloat() * 3.99f);
		else
			particles[i].image = (int)(Random::getGlobal().getRandomFloat() * 5.99f);
		if(Random::getGlobal().getRandomFloat() > 0.25f)
			particles[i].delay = (int)(Random::getGlobal().getRandomFloat() * 29.99f);
		else
			particles[i].delay = 0;
		particles[i].fade = 4.0f;
	}
}

//------------------------------------------------------------------------------
SanterExplosion::~SanterExplosion()
{

}

//------------------------------------------------------------------------------
bool SanterExplosion::draw(Drawer *d, const TwoFloats& playerPos)
{
	int i;
	float alpha;
	TwoFloats topLeft, bottomRight;

	if(!images[0])
	{
		ImageData data;
		ImageLoader imLoad;
		OpenGLDrawer *openGLD = dynamic_cast<OpenGLDrawer *>(d);

		if(openGLD)
			openGLD->setTexInterp(false);

		data = imLoad.loadImage(Images::santermess0_png, Images::santermess0_pngSize);
		images[0] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::santermess1_png, Images::santermess1_pngSize);
		images[1] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::santermess2_png, Images::santermess2_pngSize);
		images[2] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::santermess3_png, Images::santermess3_pngSize);
		images[3] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::santermess4_png, Images::santermess4_pngSize);
		images[4] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::santermess5_png, Images::santermess5_pngSize);
		images[5] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::snowflake0_png, Images::snowflake0_pngSize);
		snowflakeImages[0] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::snowflake1_png, Images::snowflake1_pngSize);
		snowflakeImages[1] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::snowflake2_png, Images::snowflake2_pngSize);
		snowflakeImages[2] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::snowflake3_png, Images::snowflake3_pngSize);
		snowflakeImages[3] = d->loadImage(data.data, data.width, data.height);

		if(openGLD)
			openGLD->setTexInterp(true);
	}

	//Update particles' positions.
	for(i=0;i<NumParticles;++i)
	{
		if((particles[i].delay <= 0) && (particles[i].fade > 0.0f))
		{
			particles[i].pos += particles[i].dir * particles[i].speed;
			particles[i].angle += particles[i].rotSpeed;

			particles[i].fade -= 0.015f;
			if(particles[i].fade <= 0.0f)
				++numDead;
		}
		else
			--particles[i].delay;
	}

	topLeft.x = playerPos.x - 740.0f;
	topLeft.y = playerPos.y - 460.0f;
	bottomRight.x = playerPos.x + 740.0f;
	bottomRight.y = playerPos.y + 460.0f;

	//Draw the visible particles.
	if(snowflake)
	{
		for(i=0;i<NumParticles;++i)
		{
			Particle *particle = &particles[i];

			if((particle->delay <= 0) && (particle->fade > 0.0f))
			{
				if(particle->pos.isWithin(topLeft, bottomRight))
				{
					alpha = particle->fade;
					if(alpha > 1.0f)
						alpha = 1.0f;

					d->drawImageRot(snowflakeImages[particle->image],
									particle->pos.x-topLeft.x-100.0f-32.0f,
									particle->pos.y-topLeft.y-100.0f-32.0f,
									particle->angle,
									8.0f,
									8.0f,
									4.0f,
									4.0f,
									1.0f,
									1.0f,
									1.0f,
									alpha);
				}
			}
		}
	}
	else
	{
		for(i=0;i<NumParticles;++i)
		{
			Particle *particle = &particles[i];

			if((particle->delay <= 0) && (particle->fade > 0.0f))
			{
				if(particle->pos.isWithin(topLeft, bottomRight))
				{
					alpha = particle->fade;
					if(alpha > 1.0f)
						alpha = 1.0f;

					d->drawImageRot(images[particle->image],
									particle->pos.x-topLeft.x-100.0f-32.0f,
									particle->pos.y-topLeft.y-100.0f-32.0f,
									particle->angle,
									8.0f,
									8.0f,
									4.0f,
									4.0f,
									1.0f,
									1.0f,
									1.0f,
									alpha);
				}
			}
		}
	}

	return (numDead >= NumParticles);
}
