//	SanterExplosion.h - Class representing the explosion of a Santer.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef SANTEREXPLOSION_H_
#define SANTEREXPLOSION_H_

#include "TwoFloats.h"

#include <stdint.h>

class Drawer;

///	Class representing the explosion of a Santer.
class SanterExplosion
{
  public:
	///	Constructor.
	SanterExplosion(const TwoFloats& pos, bool snowflakeExplosion = false);
	///	Destructor.
	~SanterExplosion();

	///	Draws the explosion.
	/*!
		\return Returns true if this explosion can now be deleted.
	 */
	bool draw(Drawer *d, const TwoFloats& playerPos);
  private:
	///	Represents a single particle.
	struct Particle
	{
		///	The particle's position.
		TwoFloats pos;
		///	The particle's direction.
		TwoFloats dir;
		///	The particle's speed.
		float speed;
		///	The particle's rotation.
		float angle;
		///	The particle's speed of rotation.
		float rotSpeed;
		///	Which image the particle is using.
		int image;
		///	Used to ensure the particles don't all appear at once.
		int delay;
		///	Used to fade the particles out after a period.
		float fade;
	};

	///	The 6 particle images.
	static uint32_t images[6];
	///	The 4 snowflake images.
	static uint32_t snowflakeImages[4];

	///	How many particles we have.
	enum
	{
		NumParticles = 32
	};

	///	The position of the explosion.
	TwoFloats position;
	///	The particles.
	Particle particles[NumParticles];
	///	Used to determine when the explosion can be deleted.
	int numDead;

	///	If this is a snowflake explosion.
	bool snowflake;
};

#endif
