//	ShockWave.cpp - Class representing a shock wave.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "ShockWave.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "NiallsDrawingLib/OpenGLDrawer.h"
#include "Images.h"
#include "MiscHelper.h"

//------------------------------------------------------------------------------
uint32_t ShockWave::image = 0;

//------------------------------------------------------------------------------
ShockWave::ShockWave(const TwoFloats& pos):
position(pos),
size(0.0f),
colourInterp(0.0f)
{
	//Set up shock wave colours.
	colours[0] = ThreeFloats(1.0f, 0.0f, 0.0f);
	colours[1] = ThreeFloats(1.0f, 0.5f, 0.0f);
	colours[2] = ThreeFloats(1.0f, 1.0f, 0.0f);
	colours[3] = ThreeFloats(0.0f, 0.5f, 0.0f);
	colours[4] = ThreeFloats(0.0f, 0.0f, 1.0f);
	colours[5] = ThreeFloats(0.5f, 0.0f, 1.0f);
	colours[6] = ThreeFloats(1.0f, 0.0f, 1.0f);
}

//------------------------------------------------------------------------------
ShockWave::~ShockWave()
{

}

//------------------------------------------------------------------------------
bool ShockWave::draw(Drawer *d, const TwoFloats& playerPos)
{
	float tempf;
	int prev, next;
	ThreeFloats col;
	bool retval = false;
	TwoFloats topLeft, bottomRight;

	if(!image)
	{
		ImageData data;
		ImageLoader imLoad;
		OpenGLDrawer *openGLD = dynamic_cast<OpenGLDrawer *>(d);

		if(openGLD)
			openGLD->setTexInterp(false);

		data = imLoad.loadImage(Images::shockwave_png, Images::shockwave_pngSize);
		image = d->loadImage(data.data, data.width, data.height);

		if(openGLD)
			openGLD->setTexInterp(false);
	}

	topLeft.x = playerPos.x - 640.0f - (32.0f * 12.0f);
	topLeft.y = playerPos.y - 360.0f - (32.0f * 12.0f);
	bottomRight.x = playerPos.x + 640.0f + (32.0f * 12.0f);
	bottomRight.y = playerPos.y + 360.0f + (32.0f * 12.0f);

	if(position.isWithin(topLeft, bottomRight))
	{
		prev = (int)colourInterp;
		next = (prev+1)%7;
		tempf = colourInterp - (float)prev;
		col = (1.0f-tempf) * colours[prev];
		col += tempf * colours[next];

		topLeft += (32.0f * 12.0f);
		d->drawImage(image,
					 position.x - topLeft.x - (size * (32.0f * 12.0f)),
					 position.y - topLeft.y - (size * (32.0f * 12.0f)),
					 12.0f * size,
					 12.0f * size,
					 col.x,
					 col.y,
					 col.z,
					 fakeLog(1.0f-size));
	}

	colourInterp += 0.2f;
	if(colourInterp >= 7.0f)
		colourInterp -= 7.0f;

	size += 0.01f;
	if(size >= 1.0f)
		retval = true;

	return retval;
}
