//	Snowball.cpp - A snowball fired by the player.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Snowball.h"
#include "NiallsDrawingLib/Drawer.h"
#include "Box2DHelpers.h"
#include "Pickup.h"

//------------------------------------------------------------------------------
Snowball::Snowball():
deathCount(2000),
dead(false)
{
	
}

//------------------------------------------------------------------------------
Snowball::~Snowball()
{
	
}

//------------------------------------------------------------------------------
void Snowball::initialise(b2World& world, const TwoFloats& pos, const TwoFloats& dir)
{
	b2BodyDef bodyDef;
	b2FixtureDef fixtureDef;
	b2CircleShape boundingBox;

	bodyDef.type = b2_dynamicBody;
	bodyDef.position.Set(pixels2Meters(pos.x), pixels2Meters(pos.y));
	physicsBody = world.CreateBody(&bodyDef);

	boundingBox.m_p.Set(0.0f, 0.0f);
	boundingBox.m_radius = pixels2Meters((8.0f-2.0f)*4.0f);
	fixtureDef.shape = &boundingBox;
	fixtureDef.density = 1.0f;
	fixtureDef.friction = 0.3f;
	fixtureDef.userData = this;
	fixtureDef.isSensor = true;
	physicsBody->CreateFixture(&fixtureDef);

	physicsBody->ApplyLinearImpulse(b2Vec2(pixels2Meters(dir.x*1000.0f),
										   pixels2Meters(dir.y*1000.0f)),
									physicsBody->GetWorldCenter());

	//physicsBody->SetTransform(bodyDef.position, 0.0f);
	//physicsBody->SetLinearDamping(0.5f);
}

//------------------------------------------------------------------------------
void Snowball::move()
{
	--deathCount;
	if(deathCount <= 0)
		dead = true;
}

//------------------------------------------------------------------------------
void Snowball::draw(Drawer *d, const TwoFloats& playerPos)
{
	TwoFloats topLeft, bottomRight;
	TwoFloats pos(b2Vec22TwoFloats(physicsBody->GetPosition()));

	topLeft.x = playerPos.x - 640.0f - (8.0f*4.0f);
	topLeft.y = playerPos.y - 360.0f - (8.0f*4.0f);
	bottomRight.x = playerPos.x + 640.0f + (8.0f*4.0f);
	bottomRight.y = playerPos.y + 360.0f + (8.0f*4.0f);

	pos.x = meters2Pixels(pos.x);
	pos.y = meters2Pixels(pos.y);
	
	if(pos.isWithin(topLeft, bottomRight))
	{
		topLeft += (8.0f*4.0f);
		d->drawImageRot(Pickup::images[Pickup::Snowball],
						pos.x-topLeft.x - (8.0f*4.0f),
						pos.y-topLeft.y - (8.0f*4.0f),
						physicsBody->GetAngle() * (180.0f/3.14159265f),
						8.0f,
						8.0f,
						4.0f,
						4.0f);
	}
}

//------------------------------------------------------------------------------
TwoFloats Snowball::getPosition() const
{
	TwoFloats retval;
	b2Vec2 physicsPos = physicsBody->GetPosition();

	retval.x = meters2Pixels(physicsPos.x);
	retval.y = meters2Pixels(physicsPos.y);

	return retval;
}

//------------------------------------------------------------------------------
void Snowball::nebulaCollide(bool val)
{
	if(val)
	{
		physicsBody->SetLinearDamping(2.0f);
		physicsBody->SetAngularDamping(2.0f);
	}
	else
	{
		physicsBody->SetLinearDamping(0.5f);
		physicsBody->SetAngularDamping(0.05f);
	}
}

//------------------------------------------------------------------------------
void Snowball::setDead()
{
	dead = true;
}
