//	Snowball.h - A snowball fired by the player.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef SNOWBALL_H_
#define SNOWBALL_H_

#include "Collidable.h"
#include "TwoFloats.h"

#include "Box2D/Box2D.h"

class Drawer;

///	A snowball fired by the player.
class Snowball : public Collidable
{
  public:
	///	Constructor.
	Snowball();
	///	Destructor.
	~Snowball();

	///	Initialises all the physics stuff.
	void initialise(b2World& world, const TwoFloats& pos, const TwoFloats& dir);

	///	Used to ensure the Snowball doesn't last forever if it doesn't hit something.
	void move();
	///	Draws the Snowball.
	void draw(Drawer *d, const TwoFloats& playerPos);

	///	Returns the Snowball's position.
	TwoFloats getPosition() const;
	///	Returns the Snowball's Box2D body.
	b2Body *getBody() {return physicsBody;};

	///	Called when the Snowball enters/leaves a nebula.
	void nebulaCollide(bool val);

	///	Used to delete the Snowball.
	void setDead();
	///	Used to delete the Snowball.
	bool getDead() const {return dead;};

	///	For collisions.
	const std::string getName() const {return "Snowball";};
  private:
	///	The Snowball's Box2D body.
	b2Body *physicsBody;

	///	Used to delete the Snowball.
	int deathCount;
	///	Used to delete the Snowball.
	bool dead;
};

#endif
