//	Snowflakes.cpp - Class to display snowflakes in the foreground.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Snowflakes.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "NiallsDrawingLib/OpenGLDrawer.h"
#include "Images.h"
#include "Random.h"

//------------------------------------------------------------------------------
uint32_t Snowflakes::images[4] = {0, 0, 0, 0};

//------------------------------------------------------------------------------
Snowflakes::Snowflakes():
beat(0.0f)
{
	int i;

	for(i=0;i<NumSnowflakes;++i)
	{
		snowflakes[i].position.x = Random::getGlobal().getRandomFloat() * 1280.0f;
		snowflakes[i].position.y = -(Random::getGlobal().getRandomFloat() * 720.0f);
		if(Random::getGlobal().getRandomFloat() > 0.25f)
			snowflakes[i].image = 0;
		else
			snowflakes[i].image = (int)(Random::getGlobal().getRandomFloat() * 3.99f);
		snowflakes[i].xIndex = Random::getGlobal().getRandomFloat();
		snowflakes[i].xSpeed = (Random::getGlobal().getRandomFloat() * 2.0f);
		snowflakes[i].ySpeed = 0.1f + Random::getGlobal().getRandomFloat() * 2.0f;
		snowflakes[i].angle = Random::getGlobal().getRandomFloat() * 359.99f;
		snowflakes[i].rotSpeed = (Random::getGlobal().getRandomFloat() * 4.0f) - 2.0f;
	}
}

//------------------------------------------------------------------------------
Snowflakes::~Snowflakes()
{

}

//------------------------------------------------------------------------------
void Snowflakes::draw(Drawer *d)
{
	int i;
	TwoFloats pos;
	Snowflake *snowflake;
	float scale = 1.0f + beat;

	if(!images[0])
	{
		ImageData data;
		ImageLoader imLoad;
		OpenGLDrawer *openGLD = dynamic_cast<OpenGLDrawer *>(d);

		if(openGLD)
			openGLD->setTexInterp(false);

		data = imLoad.loadImage(Images::snowflake0_png, Images::snowflake0_pngSize);
		images[0] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::snowflake1_png, Images::snowflake1_pngSize);
		images[1] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::snowflake2_png, Images::snowflake2_pngSize);
		images[2] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::snowflake3_png, Images::snowflake3_pngSize);
		images[3] = d->loadImage(data.data, data.width, data.height);

		if(openGLD)
			openGLD->setTexInterp(true);
	}

	for(i=0;i<NumSnowflakes;++i)
	{
		snowflake = &(snowflakes[i]);

		pos = snowflake->position;
		pos.x += sinf(snowflake->xIndex * 6.283185f) * 32.0f;

		d->drawImageRot(images[snowflake->image],
						pos.x - (3.5f*4.0f*scale),
						pos.y - (3.5f*4.0f*scale),
						snowflake->angle,
						3.5f,
						3.5f,
						4.0f*scale,
						4.0f*scale);

		snowflake->position.y += snowflake->ySpeed;
		if(snowflake->position.y > 727.0f)
			snowflake->position.y = -16.0f;
		snowflake->xIndex += snowflake->xSpeed * 0.005f;
		if(snowflake->xIndex > 1.0f)
			snowflake->xIndex -= 1.0f;
		snowflake->angle += snowflake->rotSpeed;
	}

	if(beat > 0.0f)
		beat -= 0.1f;
}

//------------------------------------------------------------------------------
void Snowflakes::setBeat()
{
	beat = 1.0f;
}
