//	Snowflakes.h - Class to display snowflakes in the foreground.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef SNOWFLAKES_H_
#define SNOWFLAKES_H_

#include "TwoFloats.h"

#include <stdint.h>

class Drawer;

///	Class to display snowflakes in the foreground.
class Snowflakes
{
  public:
	///	Constructor.
	Snowflakes();
	///	Destructor.
	~Snowflakes();

	///	Draws the snowflakes.
	void draw(Drawer *d);

	///	So we scale the snowflakes on the beat.
	void setBeat();
  private:
	///	Simple struct representing a single snowflake.
	struct Snowflake
	{
		///	The snowflake's current position.
		TwoFloats position;
		///	The image this snowflake uses.
		int image;
		///	The snowflake's sin index.
		float xIndex;
		///	The snowflake's sin speed.
		float xSpeed;
		///	The snowflake's fall speed.
		float ySpeed;
		///	The snowflake's rotational angle.
		float angle;
		///	The snowflake's rotational speed.
		float rotSpeed;
	};

	///	How many snowflakes there can be at any one time.
	enum
	{
		NumSnowflakes = 192
	};

	///	The snowflake images.
	static uint32_t images[4];

	///	The snowflakes.
	Snowflake snowflakes[NumSnowflakes];

	///	So we scale the snowflakes on the beat.
	float beat;
};

#endif
