//	StarParticles.cpp - Class to handle star particles.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "StarParticles.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "NiallsDrawingLib/OpenGLDrawer.h"
#include "Images.h"
#include "Random.h"

using namespace std;

//------------------------------------------------------------------------------
uint32_t StarParticles::starBack = 0;
uint32_t StarParticles::starFront = 0;
uint32_t StarParticles::skull = 0;

//------------------------------------------------------------------------------
StarParticles::StarParticles():
fade(true),
skulls(false)
{

}

//------------------------------------------------------------------------------
StarParticles::~StarParticles()
{

}

//------------------------------------------------------------------------------
void StarParticles::draw(Drawer *d, const TwoFloats& playerPos)
{
	int i;
	TwoFloats topLeft, bottomRight;

	if(!starBack)
	{
		ImageData data;
		ImageLoader imLoad;
		OpenGLDrawer *openGLD = dynamic_cast<OpenGLDrawer *>(d);

		if(openGLD)
			openGLD->setTexInterp(false);

		data = imLoad.loadImage(Images::starback_png, Images::starback_pngSize);
		starBack = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::starnew_png, Images::starnew_pngSize);
		starFront = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::skull_png, Images::skull_pngSize);
		skull = d->loadImage(data.data, data.width, data.height);

		if(openGLD)
			openGLD->setTexInterp(true);
	}

	topLeft.x = playerPos.x - 740.0f;
	topLeft.y = playerPos.y - 460.0f;
	bottomRight.x = playerPos.x + 740.0f;
	bottomRight.y = playerPos.y + 460.0f;

	for(i=(stars.size()-1);i>=0;--i)
	{
		if(stars[i].position.isWithin(topLeft, bottomRight))
		{
			/*d->drawImageRot(starBack,
							stars[i].position.x-topLeft.x-100.0f-16.0f,
							stars[i].position.y-topLeft.y-100.0f-16.0f,
							stars[i].angle,
							8.0f,
							8.0f,
							2.0f,
							2.0f,
							1.0f,
							1.0f,
							1.0f,
							stars[i].alpha);*/
							
			if(skulls)
			{
				d->drawImageRot(skull,
								stars[i].position.x-topLeft.x-100.0f-16.0f,
								stars[i].position.y-topLeft.y-100.0f-16.0f,
								stars[i].angle,
								8.0f,
								8.0f,
								4.0f,
								4.0f,
								1.0f,
								1.0f,
								1.0f,
								stars[i].alpha);
			}
			else
			{
					d->drawImageRot(starFront,
								stars[i].position.x-topLeft.x-100.0f-16.0f,
								stars[i].position.y-topLeft.y-100.0f-16.0f,
								stars[i].angle,
								8.0f,
								8.0f,
								2.0f,
								2.0f,
								stars[i].red,
								stars[i].green,
								stars[i].blue,
								stars[i].alpha);
			}
		}

		stars[i].angle += stars[i].rotSpeed;
		if(fade)
		{
			stars[i].alpha -= 0.0025f;
			if(stars[i].alpha <= 0.0f)
				stars.erase(stars.begin()+i);
		}
	}
}

//------------------------------------------------------------------------------
void StarParticles::addStar(const TwoFloats& pos)
{
	int colIndex;

	stars.push_back(Star());

	Star& star = stars[stars.size()-1];

	star.position = pos;
	star.angle = Random::getGlobal().getRandomFloat() * 359.0f;
	star.rotSpeed = 1.0f + (Random::getGlobal().getRandomFloat() * 2.0f);
	if(Random::getGlobal().getRandomFloat() > 0.5f)
		star.rotSpeed = -star.rotSpeed;

	colIndex = (int)(Random::getGlobal().getRandomFloat() * 5.99f);
	switch(colIndex)
	{
		case 0:
			star.red = 1.0f;
			star.green = 0.0f;
			star.blue = 0.0f;
			break;
		case 1:
			star.red = 0.0f;
			star.green = 1.0f;
			star.blue = 0.0f;
			break;
		case 2:
			star.red = 0.0f;
			star.green = 0.0f;
			star.blue = 1.0f;
			break;
		case 3:
			star.red = 1.0f;
			star.green = 1.0f;
			star.blue = 0.0f;
			break;
		case 4:
			star.red = 1.0f;
			star.green = 0.5f;
			star.blue = 0.0f;
			break;
		case 5:
			star.red = 1.0f;
			star.green = 0.0f;
			star.blue = 1.0f;
			break;
	}
	star.alpha = 1.0f;
}

//------------------------------------------------------------------------------
void StarParticles::setFade(bool val)
{
	fade = val;
}

//------------------------------------------------------------------------------
void StarParticles::setSkulls(bool val)
{
	skulls = val;
}
