//	StarParticles.h - Class to handle star particles.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef STARPARTICLES_H_
#define STARPARTICLES_H_

#include "TwoFloats.h"

#include <stdint.h>
#include <vector>

class Drawer;

///	Class to handle star particles.
class StarParticles
{
  public:
	///	Constructor.
	StarParticles();
	///	Destructor.
	~StarParticles();

	///	Draws any active star particles.
	void draw(Drawer *d, const TwoFloats& playerPos);

	///	Adds a new star at the specified position.
	void addStar(const TwoFloats& pos);

	///	Makes the stars not fade.
	void setFade(bool val);
	///	Sets the stars to draw skulls instead.
	void setSkulls(bool val);
  private:
	///	Struct represeting a single star.
	struct Star
	{
		///	The star's position.
		TwoFloats position;
		///	The star's rotation.
		float angle;
		///	The star's speed of rotation.
		float rotSpeed;
		///	The star's colour.
		float red;
		///	The star's colour.
		float green;
		///	The star's colour.
		float blue;
		///	The star's colour.
		float alpha;
	};

	///	The star back image.
	static uint32_t starBack;
	///	The star front image.
	static uint32_t starFront;
	///	The skull image.
	static uint32_t skull;

	///	All the currently active stars.
	std::vector<Star> stars;

	///	Whether to fade the stars or not.
	bool fade;
	///	Whether to draw skulls instead.
	bool skulls;
};

#endif
