//	Stars.cpp - Class to display stars in the background.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Stars.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "NiallsDrawingLib/OpenGLDrawer.h"
#include "Images.h"
#include "Random.h"

//------------------------------------------------------------------------------
uint32_t Stars::image = 0;
uint32_t Stars::corona = 0;

//------------------------------------------------------------------------------
Stars::Stars()
{
	int i;

	for(i=0;i<NumStars;++i)
	{
		stars[i].position.x = Random::getGlobal().getRandomFloat() * 1480.0f;
		stars[i].position.x += 64000.0f - 740.0f;

		stars[i].position.y = Random::getGlobal().getRandomFloat() * 920.0f;
		stars[i].position.y += 32000.0f - 460.0f;

		if(Random::getGlobal().getRandomFloat() > 0.8f)
			stars[i].image = (int)(Random::getGlobal().getRandomFloat() * 3.99f);
		else
			stars[i].image = 1;

		stars[i].fade = Random::getGlobal().getRandomFloat() * 3.14159265f;

		getRandomColour(stars[i]);
	}
}

//------------------------------------------------------------------------------
Stars::~Stars()
{

}

//------------------------------------------------------------------------------
void Stars::draw(Drawer *d, const TwoFloats& playerPos)
{
	int i;
	float tempScale = 4.0f + (4.0f * scale);
	TwoFloats topLeft, bottomRight;

	if(!image)
	{
		ImageData data;
		ImageLoader imLoad;
		OpenGLDrawer *openGLD = dynamic_cast<OpenGLDrawer *>(d);

		if(openGLD)
			openGLD->setTexInterp(false);

		data = imLoad.loadImage(Images::stars_png, Images::stars_pngSize);
		image = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::corona_png, Images::corona_pngSize);
		corona = d->loadImage(data.data, data.width, data.height);

		if(openGLD)
			openGLD->setTexInterp(true);
	}

	topLeft.x = playerPos.x - 640.0f - (2.0f*4.0f);
	topLeft.y = playerPos.y - 360.0f - (3.0f*4.0f);
	bottomRight.x = playerPos.x + 640.0f;
	bottomRight.y = playerPos.y + 360.0f;

	d->setBlendMode(Drawer::Additive);

	for(i=0;i<NumStars;++i)
	{
		if(stars[i].position.isWithin(topLeft, bottomRight))
		{
			d->drawSubImage(image,
							stars[i].position.x - topLeft.x - (2.0f*tempScale),
							stars[i].position.y - topLeft.y - (3.0f*tempScale),
							(float)stars[i].image * 0.25f,
							0.0f,
							0.25f,
							1.0f,
							tempScale,
							tempScale,
							stars[i].red,
							stars[i].green,
							stars[i].blue,
							sinf(stars[i].fade) * 0.75f);
			d->drawImage(corona,
						 stars[i].position.x - topLeft.x - (7.0f*(tempScale-1.0f)),
						 stars[i].position.y - topLeft.y - (7.0f*(tempScale-1.0f)),
						 tempScale-1.0f,
						 tempScale-1.0f,
						 stars[i].red,
						 stars[i].green,
						 stars[i].blue,
						 sinf(stars[i].fade) * 0.25f);
		}
		else
		{
			if(!stars[i].position.isWithin(topLeft-100.0f, bottomRight+100.0f))
			{
				int quadrant = (int)(Random::getGlobal().getRandomFloat() * 3.99f);

				switch(quadrant)
				{
					case 0: //Left.
						stars[i].position.x = Random::getGlobal().getRandomFloat() * 100.0f;
						stars[i].position.x += playerPos.x - 740.0f;

						stars[i].position.y = Random::getGlobal().getRandomFloat() * 920.0f;
						stars[i].position.y += playerPos.y - 460.0f;
						break;
					case 1: //Top.
						stars[i].position.x = Random::getGlobal().getRandomFloat() * 1480.0f;
						stars[i].position.x += playerPos.x - 740.0f;

						stars[i].position.y = Random::getGlobal().getRandomFloat() * 100.0f;
						stars[i].position.y += playerPos.y - 460.0f;
						break;
					case 2: //Right.
						stars[i].position.x = Random::getGlobal().getRandomFloat() * 100.0f;
						stars[i].position.x += playerPos.x + 640.0f;

						stars[i].position.y = Random::getGlobal().getRandomFloat() * 920.0f;
						stars[i].position.y += playerPos.y - 460.0f;
						break;
					case 3: //Bottom.
						stars[i].position.x = Random::getGlobal().getRandomFloat() * 1480.0f;
						stars[i].position.x += playerPos.x - 740.0f;

						stars[i].position.y = Random::getGlobal().getRandomFloat() * 100.0f;
						stars[i].position.y += playerPos.y + 360.0f;
						break;
				}
			}
		}

		stars[i].fade += 0.005f;
		if(stars[i].fade > 3.14159265f)
		{
			stars[i].position.x = Random::getGlobal().getRandomFloat() * 1480.0f;
			stars[i].position.x += playerPos.x - 740.0f;

			stars[i].position.y = Random::getGlobal().getRandomFloat() * 920.0f;
			stars[i].position.y += playerPos.y - 460.0f;

			stars[i].fade = 0.0f;
		}
	}

	d->setBlendMode(Drawer::AlphaBlend);
}

//------------------------------------------------------------------------------
void Stars::getRandomColour(Star& star)
{
	int index = (int)(Random::getGlobal().getRandomFloat() * 11.99f);

	switch(index)
	{
		case 0: //Red.
			star.red = 1.0f;
			star.green = 0.0f;
			star.blue = 0.0f;
			break;
		case 1: //Green.
			star.red = 0.0f;
			star.green = 1.0f;
			star.blue = 0.0f;
			break;
		case 2: //Blue.
			star.red = 0.0f;
			star.green = 0.0f;
			star.blue = 1.0f;
			break;
		case 3: //Yellow.
			star.red = 1.0f;
			star.green = 1.0f;
			star.blue = 0.0f;
			break;
		case 4: //Orange.
			star.red = 1.0f;
			star.green = 0.5f;
			star.blue = 0.0f;
			break;
		case 5: //Pink.
			star.red = 0.75f;
			star.green = 0.0f;
			star.blue = 1.0f;
			break;
		default:
			star.red = 1.0f;
			star.green = 1.0f;
			star.blue = 1.0f;
	}
}

//------------------------------------------------------------------------------
void Stars::setScale(float val)
{
	scale = val;
}
