//	Stars.h - Class to display stars in the background.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef STARS_H_
#define STARS_H_

#include "TwoFloats.h"

#include <stdint.h>

class Drawer;

///	Class to display stars in the background.
class Stars
{
  public:
	///	Constructor.
	Stars();
	///	Destructor.
	~Stars();

	///	Draws the stars and updates their position if necessary.
	void draw(Drawer *d, const TwoFloats& playerPos);

	///	Used to scale the stars on the beat.
	void setScale(float val);
  private:
	///	The stars' image.
	static uint32_t image;
	///	The corona image.
	static uint32_t corona;

	///	Simple struct representing a single star.
	struct Star
	{
		///	The star's position.
		TwoFloats position;
		///	Which image the star uses.
		int image;
		///	Used to fade the star in and out.
		float fade;

		///	The star's colour.
		float red;
		///	The star's colour.
		float green;
		///	The star's colour.
		float blue;
	};
	///	How many active stars we have.
	enum
	{
		NumStars = 192
	};
	///	Helper method to generate a useful random colour.
	void getRandomColour(Star& star);

	///	The active stars.
	Star stars[NumStars];

	///	Used to scale the stars on the beat.
	float scale;
};

#endif
