//	TextScroller.cpp - An amiga-esque text scroller.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "TextScroller.h"

#include <cassert>

using namespace std;

//------------------------------------------------------------------------------
TextScroller::TextScroller(const string& t):
character(0),
numChars(0),
charTimer(1.0f)
{
	unsigned int i;

	for(i=0;i<t.length();++i)
	{
		string tempstr(1, t[i]);
		text.push_back(new ScrollerChar(tempstr));
	}
}

//------------------------------------------------------------------------------
TextScroller::~TextScroller()
{
	unsigned int i;

	for(i=0;i<text.size();++i)
		delete text[i];
}

//------------------------------------------------------------------------------
void TextScroller::draw(Drawer *d, uint32_t font, float y, float colInterp)
{
	unsigned int i;
	ThreeFloats left;
	ThreeFloats right;

	if((charTimer != -1.0f) && (charTimer >= 1.0f) && ((numChars < 41) || (numChars < (int)text.size())))
	{
		++numChars;
		if((numChars == 41) || (numChars == text.size()))
			charTimer = -1.0f;
		else
			charTimer = 0.0f;
	}
	else if(charTimer != -1.0f)
		charTimer += 0.033f;

	left = (1.0f-colInterp) * colours[0];
	left += colInterp * colours[1];

	right = (1.0f-colInterp) * colours[1];
	right += colInterp * colours[2];

	for(i=character;i<(unsigned)(character+numChars);++i)
	{
		assert(i < text.size());

		if(text[i]->draw(d, font, y, left, right))
		{
			++character;
			if((character + numChars) > (int)text.size())
				--numChars;
		}
	}
}

//------------------------------------------------------------------------------
void TextScroller::setColour(const ThreeFloats& col)
{
	ThreeFloats black;

	colours[0] = colours[1];
	colours[1] = colours[2];
	colours[2] = col;
}
