//	TitleScreen.cpp - The title screen.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "TitleScreen.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "NiallsDrawingLib/OpenGLDrawer.h"
#include "Images.h"
#include "Random.h"
#include "Application.h"

#include <ctime>

//------------------------------------------------------------------------------
uint32_t TitleScreen::backImages[3] = {0, 0, 0};
uint32_t TitleScreen::titleText[4] = {0, 0, 0, 0};
uint32_t TitleScreen::donkeyImage = 0;
uint32_t TitleScreen::keyImage = 0;
uint32_t TitleScreen::winterImage = 0;
uint32_t TitleScreen::font = 0;

//------------------------------------------------------------------------------
TitleScreen::TitleScreen(Application *app):
GameState(app),
text("A game by Niall Moody (2011)        \
This synth sound reminds me a lot of the amiga games I grew up with, so I figured... Text Scroller!    \
This font is Mars Needs Cunnilingus by Anna Anthropy: www.auntiepixelante.com    \
Everything else I did myself, with the aid of the SDL, Box2D, and PortAudio libraries    \
Get more of my games at www.niallmoody.com    \
Remember kids: there is no such thing as monster evil space santer, \
and he's definitely not out there, flinging himself through space, \
looking for donkeys to punch    \
Rainbow space donkeys though, are absolutely real    \
There are even tame ones you can ride, down at the space beach    \
Just remember to wear your space hat so you don't get space sunburn        \
That's all folks    \
Have a merry winterval, and watch out for santers..."),
colInterp(0.0f),
backCount(10),
backIndex(0),
donkeyScale(0.0f),
donkeyRotate(0.0f),
starCount(15)
{
	int i;

	nextCol[0].x = 1.0f;
	updateCol();
	updateCol();
	updateCol();

	for(i=0;i<128;++i)
	{
		TwoFloats pos;

		pos.x = Random::getGlobal().getRandomFloat() * 1280.0f;
		pos.y = Random::getGlobal().getRandomFloat() * 720.0f;
		stars.addStar(pos);
	}
	stars.setFade(false);
}

//------------------------------------------------------------------------------
TitleScreen::~TitleScreen()
{

}

//------------------------------------------------------------------------------
void TitleScreen::draw(Drawer *d)
{
	int i;

	if(!backImages[0])
	{
		ImageData data;
		ImageLoader imLoad;
		OpenGLDrawer *openGLD = dynamic_cast<OpenGLDrawer *>(d);

		data = imLoad.loadImage(Images::titleback0_png, Images::titleback0_pngSize);
		backImages[0] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::titleback1_png, Images::titleback1_pngSize);
		backImages[1] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::titleback2_png, Images::titleback2_pngSize);
		backImages[2] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::titletitle0_png, Images::titletitle0_pngSize);
		titleText[0] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::titletitle1_png, Images::titletitle1_pngSize);
		titleText[1] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::titletitle2_png, Images::titletitle2_pngSize);
		titleText[2] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::titletitle3_png, Images::titletitle3_pngSize);
		titleText[3] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::titlekey_png, Images::titlekey_pngSize);
		keyImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::titlewinter_png, Images::titlewinter_pngSize);
		winterImage = d->loadImage(data.data, data.width, data.height);

		font = d->loadFont("MarsNeedsCunnilingus.ttf", 48.0f);

		if(openGLD)
			openGLD->setTexInterp(false);

		data = imLoad.loadImage(Images::donkey_png, Images::donkey_pngSize);
		donkeyImage = d->loadImage(data.data, data.width, data.height);

		if(openGLD)
			openGLD->setTexInterp(true);
	}

	//Draw the background.
	for(i=0;i<3;++i)
	{
		ThreeFloats col;
		int index = (backIndex+i)%3;

		col = (1.0f-colInterp) * prevCol[index];
		col += colInterp * nextCol[index];

		d->drawImage(backImages[i], 0, 0, 1.0f, 1.0f, col.x, col.y, col.z);
		d->drawImageRot(backImages[i],
						1280.0f,
						720.0f,
						180.0f,
						0.0f,
						0.0f,
						1.0f,
						1.0f,
						col.x,
						col.y,
						col.z);
	}

	//Draw the stars.
	stars.draw(d, TwoFloats(640.0f, 360.0f));

	//Draw the donkey.
	{
		float size;
		float rot;

		size = 3.0f + ((sinf(donkeyScale) + 1.0f) * 2.0f);
		rot = sinf(donkeyRotate) * 22.5f;

		d->drawImageRot(donkeyImage,
						640.0f-(size*32.0f),
						360.0f-(size*32.0f),
						90.0f + rot,
						32.0f,
						32.0f,
						size,
						size);
	}

	//Draw the title.
	d->drawImage(titleText[0], 0, 32);
	d->drawImage(titleText[1], 640, 32);
	{
		float tempf;
		ThreeFloats col;

		col = (1.0f-colInterp) * prevCol[0];
		col += colInterp * nextCol[0];
		tempf = col.z;
		col.z = col.y;
		col.y = col.x;
		col.x = tempf;

		d->drawImage(titleText[2], 0, 64, 1.0f, 1.0f, col.x, col.y, col.z);
		d->drawImage(titleText[3], 640, 64, 1.0f, 1.0f, col.x, col.y, col.z);
	}

	//Draw the press a key text.
	d->drawImage(keyImage, 640-512, 720-150);

	//Draw the winter unlocked text.
	if(getApplication()->getWinterUnlocked())
		d->drawImage(winterImage, (640.0f-256.0f), (360.0f-256.0f));

	//Draw the text scroller.
	text.draw(d, font, 680, colInterp);

	colInterp += 0.005f;
	if(colInterp >= 1.0f)
	{
		colInterp = 0.0f;
		updateCol();
	}

	--backCount;
	if(backCount <= 0)
	{
		backCount = 10;
		++backIndex;
		backIndex %= 3;
	}

	donkeyScale += 0.1f;
	if(donkeyScale >= 6.283185f)
		donkeyScale = fmodf(donkeyScale, 6.283185f);
	donkeyRotate += 0.04f;
	if(donkeyRotate >= 6.283185f)
		donkeyRotate = fmodf(donkeyRotate, 6.283185f);

	/*--starCount;
	if(starCount <= 0)
	{
		TwoFloats pos;

		pos.x = Random::getGlobal().getRandomFloat() * 1280.0f;
		pos.y = Random::getGlobal().getRandomFloat() * 720.0f;
		stars.addStar(pos);

		starCount = 5;
	}*/
}

//------------------------------------------------------------------------------
void TitleScreen::keyDown(const std::string& action, const SDL_keysym& keyPressed)
{
	if(action == "Escape")
		quit();
	else if(action == "Winter")
	{
		if(getApplication()->getWinterUnlocked())
			getApplication()->setWinter(!getApplication()->getWinter());
	}
	else
	{
		if(isChristmas())
			nextState("PickupSelectScreen");
		else
			nextState("GamePlay");
	}
}

//------------------------------------------------------------------------------
void TitleScreen::updateCol()
{
	int col;
	ThreeFloats textCol;

	prevCol[0] = nextCol[0];
	prevCol[1] = nextCol[1];
	prevCol[2] = nextCol[2];

	while(nextCol[0] == prevCol[0])
	{
		col = (int)(Random::getGlobal().getRandomFloat() * 7.99f);
		switch(col)
		{
			case 0: //Red
				nextCol[0].x = 1.0f;
				nextCol[0].y = 0.0f;
				nextCol[0].z = 0.0f;
				break;
			case 1: //Bright Green
				nextCol[0].x = 0.0f;
				nextCol[0].y = 1.0f;
				nextCol[0].z = 0.0f;
				break;
			case 2: //Blue
				nextCol[0].x = 0.0f;
				nextCol[0].y = 0.0f;
				nextCol[0].z = 1.0f;
				break;
			case 3: //Pink
				nextCol[0].x = 1.0f;
				nextCol[0].y = 0.0f;
				nextCol[0].z = 1.0f;
				break;
			case 4: //Yellow
				nextCol[0].x = 1.0f;
				nextCol[0].y = 1.0f;
				nextCol[0].z = 0.0f;
				break;
			case 5: //Orange
				nextCol[0].x = 1.0f;
				nextCol[0].y = 0.5f;
				nextCol[0].z = 0.0f;
				break;
			case 6: //Green
				nextCol[0].x = 0.0f;
				nextCol[0].y = 0.66f;
				nextCol[0].z = 0.0f;
				break;
			case 7: //Purple
				nextCol[0].x = 0.5f;
				nextCol[0].y = 0.0f;
				nextCol[0].z = 1.0f;
				break;
		}
		nextCol[1] = nextCol[0] * 0.85f;
		nextCol[2] = nextCol[0] * 0.7f;
	}
	
	/*textCol.x = nextCol[0].z;
	textCol.y = nextCol[0].x;
	textCol.z = nextCol[0].y;*/
	textCol.x = 1.0f - nextCol[0].x;
	textCol.y = 1.0f - nextCol[0].y;
	textCol.z = 1.0f - nextCol[0].z;
	text.setColour(textCol);
}

//------------------------------------------------------------------------------
bool TitleScreen::isChristmas() const
{
	time_t tim;
	std::tm *timeStruct;
	bool retval = false;

	tim = time(0);
	timeStruct = gmtime(&tim);

	if((timeStruct->tm_mday == 25) && (timeStruct->tm_mon == 11))
		retval = true;

	return retval;
}
