//	WreckingBall.cpp - A wee helper wrecking ball.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "WreckingBall.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "NiallsDrawingLib/OpenGLDrawer.h"
#include "Box2DHelpers.h"
#include "Random.h"
#include "Pickup.h"
#include "Images.h"

//------------------------------------------------------------------------------
uint32_t WreckingBall::image = 0;

//------------------------------------------------------------------------------
WreckingBall::WreckingBall(b2World& world,
						   b2Body *playerBody,
						   const TwoFloats& playerPos):
colourInterp(0.0f)
{
	b2BodyDef bodyDef;
	b2FixtureDef fixtureDef;
	b2CircleShape boundingBox;
	b2RopeJointDef jointDef;
	TwoFloats pos;
	float index;

	index = Random::getGlobal().getRandomFloat() * 359.99f;
	pos.x = sinf(index) * 192.0f;
	pos.y = -cosf(index) * 192.0f;
	pos += playerPos;

	bodyDef.type = b2_dynamicBody;
	bodyDef.position.Set(pixels2Meters(pos.x), pixels2Meters(pos.y));
	physicsBody = world.CreateBody(&bodyDef);

	boundingBox.m_p.Set(0.0f, 0.0f);
	boundingBox.m_radius = pixels2Meters(8.0f * 6.0f);
	fixtureDef.shape = &boundingBox;
	fixtureDef.density = 0.5f;
	fixtureDef.friction = 0.3f;
	fixtureDef.userData = this;
	physicsBody->CreateFixture(&fixtureDef);

	jointDef.bodyA = playerBody;
	jointDef.bodyB = physicsBody;
	jointDef.localAnchorA = b2Vec2(0.0f, 1.0f);
	jointDef.localAnchorB = b2Vec2(0.0f, 0.0f);
	jointDef.maxLength = pixels2Meters(320.0f);
	jointDef.collideConnected = true;

	world.CreateJoint(&jointDef);

	//Set up chain colours.
	colours[0] = ThreeFloats(1.0f, 0.0f, 0.0f);
	colours[1] = ThreeFloats(1.0f, 0.5f, 0.0f);
	colours[2] = ThreeFloats(1.0f, 1.0f, 0.0f);
	colours[3] = ThreeFloats(0.0f, 0.5f, 0.0f);
	colours[4] = ThreeFloats(0.0f, 0.0f, 1.0f);
	colours[5] = ThreeFloats(0.5f, 0.0f, 1.0f);
	colours[6] = ThreeFloats(1.0f, 0.0f, 1.0f);
}

//------------------------------------------------------------------------------
WreckingBall::~WreckingBall()
{

}

//------------------------------------------------------------------------------
void WreckingBall::draw(Drawer *d, const TwoFloats& playerPos)
{
	TwoFloats pos(b2Vec22TwoFloats(physicsBody->GetPosition()));
	ThreeFloats col;
	int prev, next;
	float tempf;

	if(!image)
	{
		ImageData data;
		ImageLoader imLoad;
		OpenGLDrawer *openGLD = dynamic_cast<OpenGLDrawer *>(d);

		if(openGLD)
			openGLD->setTexInterp(false);

		data = imLoad.loadImage(Images::wreckingball_png, Images::wreckingball_pngSize);
		image = d->loadImage(data.data, data.width, data.height);

		if(openGLD)
			openGLD->setTexInterp(true);
	}

	prev = (int)colourInterp;
	next = (prev+1)%7;
	tempf = colourInterp - (float)prev;
	col = (1.0f-tempf) * colours[prev];
	col += tempf * colours[next];

	pos.x = meters2Pixels(pos.x);
	pos.y = meters2Pixels(pos.y);

	pos.x -= (playerPos.x - 640.0f);
	pos.y -= (playerPos.y - 360.0f);

	d->drawLine(640.0f,
				360.0f,
				pos.x,
				pos.y,
				col.x,
				col.y,
				col.z);

	d->drawImage(image,
				 pos.x-(8.0f*6.0f),
				 pos.y-(8.0f*6.0f),
				 6.0f,
				 6.0f);

	colourInterp += 0.33f;
	if(colourInterp >= 7.0f)
		colourInterp -= 7.0f;
}

//------------------------------------------------------------------------------
void WreckingBall::nebulaCollide(bool val)
{
	if(val)
	{
		physicsBody->SetLinearDamping(2.0f);
		physicsBody->SetAngularDamping(2.0f);
	}
	else
	{
		physicsBody->SetLinearDamping(0.5f);
		physicsBody->SetAngularDamping(0.05f);
	}
}
