//	WreckingBall.h - A wee helper wrecking ball.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef WRECKINGBALL_H_
#define WRECKINGBALL_H_

#include "Collidable.h"
#include "ThreeFloats.h"
#include "TwoFloats.h"

#include "Box2D/Box2D.h"
#include <stdint.h>

class Drawer;

///	A wee helper wrecking ball.
class WreckingBall : public Collidable
{
  public:
	///	Constructor.
	WreckingBall(b2World& world,
				 b2Body *playerBody,
				 const TwoFloats& playerPos);
	///	Destructor.
	~WreckingBall();

	///	Draws the WreckingBall.
	void draw(Drawer *d, const TwoFloats& playerPos);

	///	Called when the wrecking ball enters/leaves a nebula.
	void nebulaCollide(bool val);

	///	For collision purposes.
	const std::string getName() const {return "WreckingBall";};
  private:
	///	The wrecking ball image.
	static uint32_t image;

	///	The WreckingBall's Box2D body.
	b2Body *physicsBody;

	///	Used to colour cycle the chain.
	float colourInterp;
	///	Used to colour cycle the chain.
	ThreeFloats colours[7];
};

#endif
