//	MiscHelper.h - Various helpful bits and pieces.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef MISCHELPER_H_
#define MISCHELPER_H_

///	Used to try and force the compiler to inline functions.
#ifdef WIN32
#define strictinline __forceinline
#elif defined (__GNUC__)
#define strictinline inline __attribute__((always_inline))
#else
#define strictinline inline
#endif

///	Cheap fake log approximation.
strictinline float fakeLog(float val)
{
	const float logCoeff = 1.0f - (1.0f/0.75f);

	return val/(val + (logCoeff*(val-1.0f)));
}

///	Cheap fake exp approximation.
strictinline float fakeExp(float val)
{
	const float expCoeff = 1.0f - (1.0f/0.1f);

	return val/(val + (expCoeff*(val-1.0f)));
}

///	Rounding function.
strictinline int round(float val)
{
	int retval;

	retval = (int)val;
	if((val-(float)retval) > 0.5f)
		++retval;

	return retval;
}

#endif
